def profile(cmd, globals, locals, sort_order, callers):  # pragma: no cover
    # runs a command under the profiler and print profiling output at shutdown
    import os
    import profile
    import pstats
    import tempfile
    fd, fn = tempfile.mkstemp()
    try:
        profile.runctx(cmd, globals, locals, fn)
        stats = pstats.Stats(fn)
        stats.strip_dirs()
        # calls,time,cumulative and cumulative,calls,time are useful
        stats.sort_stats(*sort_order or ('cumulative', 'calls', 'time'))
        if callers:
            stats.print_callers(.3)
        else:
            stats.print_stats(.3)
    finally:
        os.remove(fn)



# -*- encoding: utf-8 -*-
"""
GObj :class:`GWSGIServer`
=========================

.. autoclass:: GWSGIServer
    :members:

"""

from ginsfsm.gobj import GObj
from ginsfsm.c_srv_sock import GServerSock
from ginsfsm.wsgi.c_httpchannel import HTTPChannel


def ac_new_httpchannel(self, event):
    """ New gsock clisvr, create a new httpchannel
    """
    self._n_connected_clisrv += 1
    gsock = event.source[-1]  # new gsock clisrv

    if self.name and len(self.name):
        prefix_name = self.name
    else:
        prefix_name = None
    self.serversock = self.create_gobj(
        prefix_name + '.http-channel' if prefix_name else None,
        HTTPChannel,
        self,
        wsgi_server=self,
        gsock=gsock,
    )


def ac_drop_httpchannel(self, event):
    """ httpchannel closed, drop it.
    """
    # TODO: need to drop gsock, httpchannel and wsgiapp
    channel = event.source[-1]
    self._n_connected_clisrv -= 1
    self.destroy_gobj(channel)


def ac_timeout(self, event):
    self.set_timeout(10)
    print "Server's clients: %d, connected %d" % (
        len(self.dl_childs), self._n_connected_clisrv)


GWSGISERVER_FSM = {
    'event_list': (
        'EV_SET_TIMER: bottom output',
        'EV_TIMEOUT: bottom input',
        'EV_CONNECTED: bottom input',
        'EV_DISCONNECTED: bottom input',
    ),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_TIMEOUT',          ac_timeout,             None),
            ('EV_CONNECTED',        ac_new_httpchannel,     None),
            ('EV_DISCONNECTED',     ac_drop_httpchannel,    None),
        ),
    }
}

GWSGISERVER_GCONFIG = {
    'host': [str, '', 0, None, "listening host"],
    'port': [int, 0, 0, None, "listening port"],
    'origins': [None, None, 0, None,
        "TODO:list of (host, port) tuples allowed to connect from"],
    #TODO: implement a multi wsgi application
    'application': [None, None, 0, None, "wsgi application"],
    'url_scheme': [str, 'http', 0, None, "default ``wsgi.url_scheme`` value"],
    'identity': [str, 'ginsfsm-wsgi', 0, None, "Server http header"],
}


class GWSGIServer(GObj):
    """  WSGI Server gobj.

    The incoming connections will create a new :class:`ginsfsm.c_sock.GSock`
    :term:`gobj`,
    that will be child of the :attr:`subscriber` :term:`gobj`.

        .. warning::  Remember destroy the accepted `gobj`
           with :func:`destroy_gobj` when the `gobj` has been disconnected.

           The `subcriber` knows when a new `gobj` has been accepted because it
           receives the ``'EV_CONNECTED'`` event.

           When the `subcriber` receives a ``'EV_DISCONNECTED'`` event must
           destroy the `gobj` because the connection ceases to exist.

    .. ginsfsm::
       :fsm: GWSGISERVER_FSM
       :gconfig: GWSGISERVER_GCONFIG

    *Input-Events:*

        The relationship is directly between the
        accepted :class:`ginsfsm.c_sock.GSock` gobj and the :attr:`subscriber`.

        See :class:`ginsfsm.c_sock.GSock` `input-events`.

    *Output-Events:*

        The relationship is directly between the
        accepted :class:`ginsfsm.c_sock.GSock` gobj and the :attr:`subscriber`.

        See :class:`ginsfsm.c_sock.GSock` `output-events`.
    """

    def __init__(self):
        GObj.__init__(self, GWSGISERVER_FSM, GWSGISERVER_GCONFIG)
        self._n_connected_clisrv = 0

    def start_up(self):
        if self.name and len(self.name):
            prefix_name = self.name
        else:
            prefix_name = None
        self.serversock = self.create_gobj(
            prefix_name + '.sock-server' if prefix_name else None,
            GServerSock,
            self,
            subscriber=self,
            host=self.host,
            port=self.port,
            origins=self.origins,
        )

        # Used in environ
        self.effective_host, self.effective_port = self.serversock.getsockname()
        self.server_name = self._get_server_name(self.host)

    def _get_server_name(self, ip):
        """Given an IP or hostname, try to determine the server name."""
        if ip:
            srv_name = str(ip)
        else:
            srv_name = str(self.serversock.socketmod.gethostname())
        # Convert to a host name if necessary.
        for c in srv_name:
            if c != '.' and not c.isdigit():
                return srv_name
        try:
            if srv_name == '0.0.0.0':
                return 'localhost'
            srv_name = self.serversock.socketmod.gethostbyaddr(srv_name)[0]
        except:
            pass
        return srv_name

    def getsockname(self):
        return self.serversock.socket.getsockname()



"""
GObj :class:`OnGSock`
=====================

Utility for check :class:`ginsfsm.c_sock.GSock`.

Connect to an url, get /, and disconnect, periodically.

.. autoclass:: OnGSock
    :members:

"""

from ginsfsm.gobj import GObj
from ginsfsm.gaplic import GAplic
from ginsfsm.c_timer import GTimer
from ginsfsm.c_sock import GSock
from ginsfsm.utils import hexdump

import logging
logging.basicConfig(level=logging.DEBUG)

QUERY = "GET / HTTP/1.1\r\n" + \
    "Host: %s\r\n" + \
    "\r\n"


def ac_connect(self, event):
    self.send_event(self.gsock, 'EV_CONNECT')
    self.set_timeout(60)


def ac_connected(self, event):
    self.clear_timeout()
    self.set_timeout(self.seconds)


def ac_transmit(self, event):
    data = QUERY % self.url
    if self.verbose:
        print('Send %s' % data)
    self.send_event(self.gsock, 'EV_SEND_DATA', data=data)


def ac_disconnect(self, event):
    self.send_event(self.gsock, 'EV_DROP')


def ac_disconnected(self, event):
    self.set_timeout(self.seconds)


def ac_rx_data(self, event):
    data = event.kw['data']
    if self.verbose:
        print('Receiving %d bytes' % len(data))
        print(hexdump('<=', data))


def ac_timeout_connect(self, event):
    self.set_new_state('ST_DISCONNECTED')
    self.set_timeout(5)

ONGSOCK_FSM = {
    'event_list': (
        'EV_SET_TIMER: bottom output',
        'EV_TIMEOUT:bottom input',
        'EV_CONNECTED:bottom input',
        'EV_DISCONNECTED:bottom input',
        'EV_RX_DATA:bottom input',
        'EV_TRANSMIT_READY:bottom input',
    ),
    'state_list': (
        'ST_DISCONNECTED',
        'ST_WAIT_CONNECTED',
        'ST_CONNECTED',
    ),
    'machine': {
        'ST_DISCONNECTED':
        (
            ('EV_TIMEOUT',          ac_connect,         'ST_WAIT_CONNECTED'),
        ),
        'ST_WAIT_CONNECTED':
        (
            ('EV_CONNECTED',        ac_connected,       'ST_CONNECTED'),
            ('EV_DISCONNECTED',     ac_disconnected,    'ST_DISCONNECTED'),
            ('EV_TIMEOUT',          ac_timeout_connect, None),
        ),
        'ST_CONNECTED':
        (
            ('EV_DISCONNECTED',     ac_disconnected,    'ST_DISCONNECTED'),
            ('EV_TIMEOUT',          ac_disconnect,      None),
            ('EV_RX_DATA',          ac_rx_data,         None),
            ('EV_TRANSMIT_READY',   ac_transmit,        None),
        ),
    }
}


ONGSOCK_GCONFIG = {  # type, default_value, flag, validate_function, desc
    'verbose': [int, 0, 0, None, "Increase output verbosity. Values [0,1,2]"],
    'seconds': [int, 5, 0, None, "Seconds to repeat the connection."],
    'url': [str, 'www.google.com', 0, None, "Url to connect."],
}


class OnGSock(GObj):
    """  OnGSock GObj.

    .. ginsfsm::
       :fsm: ONGSOCK_FSM
       :gconfig: ONGSOCK_GCONFIG

    *Input-Events:*

        * :attr:`'EV_TIMEOUT'`: Timer over.
            Start the machine.

        * :attr:`'EV_CONNECTED'`: Connected to url.

        * :attr:`'EV_DISCONNECTED'`: Disconnected from url.

        * :attr:`'EV_RX_DATA'`: Receiving data from url.

        * :attr:`'EV_TRANSMIT_READY'`: Url ready to transmit it data.


    *Output-Events:*

        * :attr:`'EV_START_TIMER'`: Start timer.

    """
    def __init__(self):
        GObj.__init__(self, ONGSOCK_FSM, ONGSOCK_GCONFIG)

    def start_up(self):
        """ Initialization zone."""
        self.timer = self.create_gobj(
            None,
            GTimer,
            self,
            timeout_event_name='EV_TIMEOUT',
        )
        self.timer.subscribe_event('EV_TIMEOUT', self)

        self.gsock = self.create_gobj(
            None,
            GSock,
            self,
            connected_event_name='EV_CONNECTED',
            disconnected_event_name='EV_DISCONNECTED',
            host=self.url,
            port=80,
        )

        if self.verbose > 1:
            settings = {
                'GObj.trace_mach': True,
                'GObj.logger': logging,
            }
            self.overwrite_parameters(-1, **settings)

        self.set_timeout(1)

    def set_timeout(self, seconds):
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=seconds)

    def clear_timeout(self):
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=-1)


if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "seconds",
        type=int,
        nargs='?', default=2,
        help="Seconds to repeat the url."
    )
    parser.add_argument(
        "url",
        nargs='?', default='www.google.com',
        help="Url to connect."
    )
    parser.add_argument(
        "-v",
        "--verbose",
        help="Increase output verbosity",
        type=int,
        choices=[0, 1, 2],
        default=1,
    )
    args = parser.parse_args()

    ga = GAplic()
    ga.create_gobj(
        'ongsock',
        OnGSock,
        None,
        verbose=args.verbose,
        seconds=args.seconds,
        url=args.url,
    )
    try:
        ga.mt_process()
    except (KeyboardInterrupt, SystemExit):
        print('Program stopped')



# -*- coding: utf-8 -*-
from .compat import text_type


def string_to_bytearray(s):
    if isinstance(s, text_type):
        s = bytearray(s, encoding="utf-8", errors="strict")
    else:
        s = bytes(s)
    return s


def string_to_pack(s):
    if isinstance(s, text_type):
        s = bytearray(s, encoding="utf-8", errors="strict")
    s = bytes(s)
    return s


HEX_FILTER = ''.join(
    [(len(repr(chr(x))) == 3) and chr(x) or '.' for x in range(256)]
    )


def hexdump(prefix, src, length=16):
    N = 0
    result = ''
    src = bytes(src)
    while src:
        s, src = src[:length], src[length:]
        hexa = ' '.join(["%02X" % ord(x) for x in s])
        s = s.translate(HEX_FILTER)
        result += "%s %04X: %-*s  %s\n" % (prefix, N, length*3, hexa, s)
        N += length
    return result



# -*- encoding: utf-8 -*-

from .parser import HTTPRequestParser
from .c_wsgiapp import WSGIApp

from ginsfsm.gobj import GObj


def ac_disconnected(self, event):
    """ httpchannel disconnected, inform to wsgi_server for to drop it.
    """
    self.post_event(self.wsgi_server, 'EV_DISCONNECTED')


def ac_httpchannel_rx_data(self, event):
    """ Received data. Can be one or more requests.
    """
    data = event.data
    if not data:
        return
    new_request = self._current_request
    requests = []
    while data:
        if new_request is None:
            new_request = HTTPRequestParser(
                url_scheme=self.wsgi_server.url_scheme,
                inbuf_overflow=self.inbuf_overflow,
                max_request_header_size=self.max_request_header_size,
                max_request_body_size=self.max_request_body_size)
        n = new_request.received(data)
        if new_request.expect_continue and new_request.headers_finished:
            # guaranteed by parser to be a 1.1 new_request
            new_request.expect_continue = False
            if not self.sent_continue:
                # there's no current task, so we don't need to try to
                # lock the outbuf to append to it.
                self.gsock.outbufs[-1].append(b'HTTP/1.1 100 Continue\r\n\r\n')
                self.sent_expect_continue = True
                self.gsock.send_some()
                new_request.completed = False
        if new_request.completed:
            # The new_request (with the body) is ready to use.
            self._current_request = None
            if not new_request.empty:
                requests.append(new_request)
            new_request = None
        else:
            self._current_request = new_request
        if n >= len(data):
            break
        data = data[n:]

    if requests:
        self.send_event(self.wsgiapp, 'EV_EXECUTE_REQUESTS', requests=requests)


def ac_timeout(self, event):
    #TODO: channel_timeout = 120 inactivity timeout
    pass


HTTPCHANNEL_FSM = {
    'event_list': ('EV_TIMEOUT', 'EV_DISCONNECTED', 'EV_RX_DATA',),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_TIMEOUT',          ac_timeout,             'ST_IDLE'),
            ('EV_DISCONNECTED',     ac_disconnected,        'ST_IDLE'),
            ('EV_RX_DATA',          ac_httpchannel_rx_data, 'ST_IDLE'),
        ),
    }
}

HTTPCHANNEL_GCONFIG = {
    'wsgi_server': [None, None, 0, None, "wsgi server"],
    'gsock': [None, None, 0, None, "clisrv gsock"],
    # A tempfile should be created if the pending input is larger than
    # inbuf_overflow, which is measured in bytes. The default is 512K.  This
    # is conservative.
    'inbuf_overflow': [int, 524288, 0, None, ""],
    # maximum number of bytes of all request headers combined (256K default)
    'max_request_header_size': [int, 262144, 0, None, ""],
    # maximum number of bytes in request body (1GB default)
    'max_request_body_size': [int, 1073741824, 0, None, ""],
}


class HTTPChannel(GObj):
    """  HTTP Channel gobj.
    """
    def __init__(self):
        GObj.__init__(self, HTTPCHANNEL_FSM, HTTPCHANNEL_GCONFIG)
        self._current_request = None  # current receiving request

    def start_up(self):
        self.gsock.delete_all_subscriptions()
        self.gsock.subscribe_event(None, self)  # take on all gsock events

        if self.name and len(self.name):
            prefix_name = self.name
        else:
            prefix_name = None
        self.wsgiapp = self.create_gobj(
            prefix_name + '.wsgi-app' if prefix_name else None,
            WSGIApp,
            self,
            wsgi_server=self.wsgi_server,
            gsock=self.gsock,
        )



# -*- encoding: utf-8 -*-
"""
GObj :class:`GSock`
===================

GObj for manage socket events.

.. autoclass:: GSock
    :members: start_up, get_next_dst

"""

# ======================================================================
#   Code inspired in asyncore.py and tornado.
# ======================================================================

import select
import socket
import time
import errno
import logging
import sys
import warnings
import os

from ginsfsm.gobj import GObj
from ginsfsm.utils import hexdump
from ginsfsm.buffers import (
    ReadOnlyFileBasedBuffer,
    OverflowableBuffer,
)
from errno import (
    EALREADY,
    EINPROGRESS,
    EWOULDBLOCK,
    ECONNRESET,
    EINVAL,
    ENOTCONN,
    ESHUTDOWN,
    EINTR,
    EISCONN,
    EBADF,
    ECONNABORTED,
    EADDRNOTAVAIL,
    errorcode
    )

_DISCONNECTED = frozenset((
    ECONNRESET,
    ENOTCONN,
    ESHUTDOWN,
    ECONNABORTED,
    ))


class IOLoop(object):
    # Constants from the epoll module
    _EPOLLIN = 0x001
    _EPOLLPRI = 0x002
    _EPOLLOUT = 0x004
    _EPOLLERR = 0x008
    _EPOLLHUP = 0x010
    _EPOLLRDHUP = 0x2000
    _EPOLLONESHOT = (1 << 30)
    _EPOLLET = (1 << 31)

    # Our events map exactly to the epoll events
    NONE = 0
    READ = _EPOLLIN
    WRITE = _EPOLLOUT
    ERROR = _EPOLLERR | _EPOLLHUP | _EPOLLRDHUP


def _strerror(err):
    try:
        return os.strerror(err)
    except (ValueError, OverflowError, NameError):
        if err in errorcode:
            return errorcode[err]
        return "Unknown error %s" % err

_reraised_exceptions = (
    KeyboardInterrupt,
    SystemExit,
    )


def readwrite(obj, flags):
    try:
        if flags & (IOLoop.ERROR):
            obj.handle_close()
            return
        if flags & (IOLoop.READ):
            obj.handle_read_event()
        if flags & (IOLoop.WRITE):
            obj.handle_write_event()
    except socket.error as e:
        if e.args[0] not in (
                EBADF,
                ECONNRESET,
                ENOTCONN,
                ESHUTDOWN,
                ECONNABORTED):
            obj.handle_error()
        else:
            obj.handle_close()
    except _reraised_exceptions:
        close_all_sockets(obj._socket_map)
        raise
    except:
        #obj.handle_error() #TODO
        raise


def poll_loop(socket_map, _impl, timeout):
    """ check poll, return True if some sock event, otherwise False.
    """
    ret = False
    try:
        event_pairs = _impl.poll(timeout)
    except Exception as e:
        # Depending on python version and IOLoop implementation,
        # different exception types may be thrown and there are
        # two ways EINTR might be signaled:
        # * e.errno == errno.EINTR
        # * e.args is like (errno.EINTR, 'Interrupted system call')
        if (getattr(e, 'errno', None) == errno.EINTR or
                (isinstance(getattr(e, 'args', None), tuple) and
                len(e.args) == 2 and e.args[0] == errno.EINTR)):
            return ret
        else:
            raise
    #print "events ------------->", event_pairs
    #print "sockmp ------------->", socket_map.keys()
    for fd, events in event_pairs:
        obj = socket_map.get(fd, None)
        if obj is None:
            continue
        ret = True
        readwrite(obj, events)
    return ret


#===========================================================
#       GSock gobj
#===========================================================
def ac_connect(self, event):
    self._mt_connect(**event.kw)


def ac_drop(self, event):
    self._mt_drop()


def ac_send_data(self, event):
    self._mt_send_data(event.data)

GSOCK_FSM = {
    'event_list': (
        'EV_CONNECT:top input',
        'EV_DROP:top input',
        'EV_SEND_DATA: top input',
        'EV_CONNECTED: top output',
        'EV_DISCONNECTED: top output',
        'EV_RX_DATA: top output',
        'EV_TRANSMIT_READY: top output'
    ),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_CONNECT',  ac_connect,     None),
            ('EV_DROP',     ac_drop,        None),
            ('EV_SEND_DATA',  ac_send_data,   None),
        ),
    }
}

GSOCK_GCONFIG = {
    'subscriber': [None, None, 0, None,
        "subcriber of all output-events."
        "Default is ``None``, i.e., the parent"
        ],
    'host': [str, '', 0, None, "server or client host (ip or name)"],
    'port': [int, 0, 0, None, "server or client port"],
    'origins': [None, None, 0, None,
        "TODO:list of (host, port) tuples allowed to connect from"],
    'tx_buffer_size': [int, 4096, 0, None, ""],
    'connected_event_name': [str, 'EV_CONNECTED', 0, None, ""],
    'disconnected_event_name': [str, 'EV_DISCONNECTED', 0, None, ""],
    'transmit_ready_event_name': [str, 'EV_TRANSMIT_READY', 0, None, ""],
    'rx_data_event_name': [str, 'EV_RX_DATA', 0, None, ""],
    '_socket_map': [None, {}, 0, None, "Set by gaplic. Dict {fd:Gobj}"],
    '_impl_poll': [None, None, 0, None, "Set by gaplic. epoll implementation"],

    # send_bytes is the number of bytes to send to socket.send().  Multiples
    # of 9000 should avoid partly-filled packets, but don't set this larger
    # than the TCP write buffer size.  In Linux, /proc/sys/net/ipv4/tcp_wmem
    # controls the minimum, default, and maximum sizes of TCP write buffers.
    'send_bytes': [int, 18000, 0, None, ""],

    # A tempfile should be created if the pending output is larger than
    # outbuf_overflow, which is measured in bytes. The default is 1MB.  This
    # is conservative.
    'outbuf_overflow': [int, 1048576, 0, None, ""],
    'close_when_flushed': [bool, False, 0, None,
        "True to close the socket when flushed"],
}


class GSock(GObj):
    """  Client socket gobj.

    Manage socket events.

    .. ginsfsm::
       :fsm: GSOCK_FSM
       :gconfig: GSOCK_GCONFIG


    *Input-Events:*
        * :attr:`'EV_SEND_DATA'`: transmit ``event.data``.

          Mandatory attributes of the received :term:`event`:

          * ``data``: data to send.

    *Output-Events:*
        * :attr:`'EV_CONNECTED'`: socket connected.

          Attributes added to the sent :term:`event`:

            * ``peername``: remote address to which the socket is connected.
            * ``sockname``: the socket’s own address.

        * :attr:`'EV_DISCONNECTED'`: socket disconnected.
        * :attr:`'EV_TRANSMIT_READY'`: socket ready to transmit more data.
        * :attr:`'EV_RX_DATA'`: data received.
          Attributes added to the sent :term:`event`:

            * ``data``: Data received from remote address.
    """
    def __init__(self):
        GObj.__init__(self, GSOCK_FSM, GSOCK_GCONFIG)

        self.txed_msgs = 0  # TODO:  estos alarmas/estadisticas
        self.rxed_msgs = 0
        self.txed_bytes = 0
        self.rxed_bytes = 0

        self.connected = False
        self.addr = None        # dont't make private: waitress use it.
        self.socket = None      # dont't make private: waitress use it.

        self._socket_map = {}   # socket_map set by gaplic. Dict {fd:Gobj}
        self._impl_poll = None  # _poll(),epoll() implementation
        self._accepting = False
        self._transmit_ready_event_done = False
        self._clisrv = False
        self._fileno = None
        self.last_activity = 0  # Time of last activity
        self.will_close = False  # set to True to close the socket.
        self.outbufs = [OverflowableBuffer(self.outbuf_overflow)]

    def start_up(self):
        """ Initialization zone.

        Subcribe all enabled :term:`output-event`'s to ``subscriber``
        with this sentence::

            self.subscribe_event(None, self.subscriber)
        """
        self.subscribe_event(None, self.subscriber)

    def set_clisrv_socket(self, sock):
        # Set to nonblocking just to make sure for cases where we
        # get a socket from a blocking source.
        sock.setblocking(0)
        self.add_socket(sock)
        self.connected = True
        self._clisrv = True
        # The constructor no longer requires that the socket
        # passed be connected.
        try:
            self.addr = sock.getpeername()
        except socket.error as err:
            if err.args[0] == ENOTCONN:
                # To handle the case where we got an unconnected
                # socket.
                self.connected = False
            else:
                # The socket is broken in some unknown way, alert
                # the user and remove it from the socket_map (to prevent
                # polling of broken sockets).
                self.remove_socket()
                raise

    def __repr__(self):
        status = [self.__class__.__module__ + "." + self.__class__.__name__]
        if self._accepting and self.addr:
            status.append('listening')
        elif self.connected:
            status.append('connected')
        if self.addr is not None:
            try:
                status.append('%s:%d' % self.addr)
            except TypeError:
                status.append(repr(self.addr))
        return '<%s at %#x>' % (' '.join(status), id(self))

    __str__ = __repr__

    def create_socket(self, family, xtype):
        self.family_and_type = family, xtype
        sock = socket.socket(family, xtype)
        sock.setblocking(0)
        self.add_socket(sock)

    def set_reuse_addr(self):
        # try to re-use a server port if possible
        try:
            self.socket.setsockopt(
                socket.SOL_SOCKET, socket.SO_REUSEADDR,
                self.socket.getsockopt(socket.SOL_SOCKET,
                                       socket.SO_REUSEADDR) | 1
                )
        except socket.error:
            pass

    def add_socket(self, sock):
        self.socket = sock
        self._fileno = sock.fileno()
        self._socket_map[self._fileno] = self
        self._impl_poll.register(
            self._fileno, IOLoop.WRITE | IOLoop.READ | IOLoop.ERROR)

    def remove_socket(self):
        fd = self._fileno
        self._socket_map.pop(fd, None)
        self._fileno = None
        self.socket = None
        try:
            self._impl_poll.unregister(fd)
        except (OSError, IOError):
            logging.error("Error deleting fd from IOLoop", exc_info=True)

    #==================================================
    #   socket object methods.
    #==================================================
    def listen(self, num):
        self._accepting = True
        if os.name == 'nt' and num > 5:
            num = 5
        return self.socket.listen(num)

    def bind(self, addr):
        self.addr = addr
        return self.socket.bind(addr)

    def connect(self, address):
        self.connected = False
        err = self.socket.connect_ex(address)
        if err in (EINPROGRESS, EALREADY, EWOULDBLOCK, ) \
        or err == EINVAL and os.name in ('nt', 'ce'):
            return
        if err in (0, EISCONN):
            self.addr = address
            self.handle_connect_event()
        else:
            raise socket.error(err, errorcode[err])

    def accept(self):
        # XXX can return either an address pair or None
        try:
            conn, addr = self.socket.accept()
        except TypeError:
            return None
        except socket.error as why:
            if why.args[0] in (EWOULDBLOCK, ECONNABORTED):
                return None
            else:
                raise
        else:
            return conn, addr

    def send(self, data):
        try:
            result = self.socket.send(data)
            return result
        except socket.error as why:
            if why.args[0] == EWOULDBLOCK:
                return 0
            elif why.args[0] in _DISCONNECTED:
                self.handle_close()
                return 0
            else:
                raise

    def recv(self):
        try:
            data = self.socket.recv(8192)
            if not data:
                # a closed connection is indicated by signaling
                # a read condition, and having recv() return 0.
                self.handle_close()
                return b''
            else:
                return data
        except socket.error as why:
            # winsock sometimes throws ENOTCONN
            if why.args[0] in _DISCONNECTED:
                self.handle_close()
                return b''
            else:
                raise

    def close(self):
        self.connected = False
        self._accepting = False
        if self.socket:
            try:
                self.socket.close()
            except socket.error as why:
                if why.args[0] not in (ENOTCONN, EBADF):
                    raise

    # cheap inheritance, used to pass all other attribute
    # references to the underlying socket object.
    def __getattr__(self, attr):
        try:
            retattr = getattr(self.socket, attr)
        except AttributeError:
            raise AttributeError("%s instance has no attribute '%s'"
                                 % (self.__class__.__name__, attr))
        else:
            msg = "%(me)s.%(attr)s is deprecated; use %(me)s.socket.%(attr)s "\
                "instead" % {'me': self.__class__.__name__, 'attr': attr}
            warnings.warn(msg, DeprecationWarning, stacklevel=2)
            return retattr

    def handle_read_event(self):
        #self.logger and self.logger.debug("handle_read_event    (%s %s)" % (self, self.name))
        if self._accepting:
            # _accepting sockets are never connected, they "spawn" new
            # sockets that are connected
            self.handle_accept()
        elif not self.connected:
            self.handle_connect_event()
            self.handle_read()
        else:
            self.handle_read()

    def handle_connect_event(self):
        #self.logger and self.logger.debug("handle_connect_event (%s %s)" % (self, self.name))
        err = self.socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
        if err != 0:
            raise socket.error(err, _strerror(err))
        self.connected = True
        self.handle_connect()

    def handle_write_event(self):
        #self.logger and self.logger.debug("handle_write_event   (%s %s)" % (self, self.name))
        if self._accepting:
            # Accepting sockets shouldn't get a write event.
            # We will pretend it didn't happen.
            return

        if not self.connected:
            #check for errors
            err = self.socket.getsockopt(socket.SOL_SOCKET, socket.SO_ERROR)
            if err != 0:
                raise socket.error(err, _strerror(err))

            self.handle_connect_event()
        self.handle_write()

    def handle_error(self):
        nil, t, v, tbinfo = compact_traceback()

        # sometimes a user repr method will crash.
        try:
            self_repr = repr(self)
        except:
            self_repr = '<__repr__(self) failed for object at %0x>' % id(self)

        logging.error(
            'uncaptured python exception, closing channel %s (%s:%s %s)' % (
                self_repr,
                t,
                v,
                tbinfo
                )
            )
        self.handle_close()

    #====================================================
    #   Mine's
    #====================================================
    def handle_accept(self):
        try:
            pair = self.accept()
            if pair is not None:
                self.handle_accepted(*pair)

        except socket.error:
            # Linux: On rare occasions we get a bogus socket back from
            # accept. socketmodule.c:makesockaddr complains that the
            # address family is unknown. We don't want the whole server
            # to shut down because of this.
            logging.warning('server accept() threw an exception', exc_info=True)

    def handle_accepted(self, sock, addr):
        sock.close()
        logging.warning('unhandled accepted event')

    def _mt_connect(self, **kw):
        """ Try to connect to (host, port).

        :param kw: valid keyword arguments are ``host`` and ``port``.

        This method calls to :meth:`get_next_dst` to get the destination tuple.

        Example::

            mt_connect(host='127.0.0.1', port=80) #override host,port attributes

        Or::

            mt_connect() # use current host,port attributes

        :class:`GSock` will broadcast some of ``'EV_CONNECTED'`` or
        ``'EV_DISCONNECTED'`` event as result of this action.
        """
        self.__dict__.update(**kw)

        if self.connected:
            logging.error(
                "ERROR connecting to host '%s', port %s. "
                "ALREADY CONNECTED." % (self.host, self.port))
            return False
        if self.socket:
            logging.error(
                "ERROR connecting to host '%s', port %s. SOCKET EXISTS." %
                (self.host, self.port))
            self.close()  # ???
            self.remove_socket()  # ???
            #TODO: si cumple el timeout de conexion viene por aqui
            #return False ???

        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        self.host, self.port = self.get_next_dst()
        try:
            self.ip = ip = socket.gethostbyname(self.host)
        except Exception as e:
            logging.error('ERROR gethostbyname %s' % e)
            return False
        logging.info("%s ==> Connecting to host '%s', ip '%s', port %s..." % (
            self.name, self.host, ip, self.port))
        try:
            self.connect((ip, self.port))
        except Exception as e:
            logging.error("_mt_connect() ERROR %s" % repr(e))
            return False
        return True

    def get_next_dst(self):
        """ Supply the destination ``(host,port)`` tuple to
        :meth:`_mt_connect` method.
        By default this function returns the internal ``(host,port)``
        attributes.

        TO BE OVERRIDE if you need other policy.
        """
        return (self.host, self.port)

    def handle_connect(self):
        data = {}
        data['peername'] = self.socket.getpeername()
        data['sockname'] = self.socket.getsockname()
        logging.info("%s <== Connected! %s host '%s'" % (
            self.name, "FROM" if self._clisrv else "TO", str(data['peername'])))
        # remove WRITE
        self._impl_poll.modify(self._fileno, IOLoop.READ | IOLoop.ERROR)
        self.broadcast_event(self.connected_event_name, **data)

    def _mt_drop(self):
        """ Drop the connexion.
        """
        logging.info(
            "%s ==> Disconnecting from host '%s', ip '%s', port %s..." % (
            self.name, self.host, self.ip, self.port))
        if self.socket:
            self.socket.shutdown(socket.SHUT_RDWR)
            self._impl_poll.modify(
                self._fileno, IOLoop.WRITE | IOLoop.READ | IOLoop.ERROR)
        else:
            self.close()
        return 0

    def handle_close(self):
        for outbuf in self.outbufs:
            try:
                outbuf._close()
            except:
                logging.exception(
                    'Unknown exception while trying to close outbuf')
        self.close()
        self.remove_socket()
        if 0:
            logging.info(
                "%s <== Disconnected! from host '%s', port %s" % (
                self.name, self.host, self.port))
        #TODO: pon la causa del disconnect
        self.broadcast_event(self.disconnected_event_name)

    def handle_read(self):
        try:
            data = self.recv()
        except socket.error:
            self.handle_error()
            return

        ln = len(data)
        self.rxed_msgs += 1
        self.rxed_bytes += ln
        if self.trace_dump:
            self.logger and self.logger.debug("Recv data '%s' (%d bytes)\n%s" % (
                self.name, ln, hexdump('<=', data)))

        self.broadcast_event(self.rx_data_event_name, data=data)

    def readable(self):
        #"predicate for inclusion in the readable for select()"
        return True

    #-------------------------------------------------#
    #               Write zone
    #-------------------------------------------------#
    def any_outbuf_has_data(self):
        for outbuf in self.outbufs:
            if bool(outbuf):
                return True
        return False

    def writable(self):
        #"predicate for inclusion in the writable for select()"
        return self.any_outbuf_has_data() or \
            (not self.connected) or \
            self.will_close

    def _mt_send_data(self, data):
        """ Send data.
        :param data: data to send.
        """
        self.write_soon(data)
        self._transmit_ready_event_done = False
        self.send_some()

    def write_soon(self, data):
        """ Same as mt_send_data. Need it for waitress use.
        """
        if data:
            if data.__class__ is ReadOnlyFileBasedBuffer:
                # they used wsgi.file_wrapper
                self.outbufs.append(data)
                nextbuf = OverflowableBuffer(self.outbuf_overflow)
                self.outbufs.append(nextbuf)
            else:
                self.outbufs[-1].append(data)
            return len(data)
        return 0

    def handle_write(self):
        # Precondition: there's data in the out buffer to be sent, or
        # there's a pending will_close request
        if not self.connected:
            # we dont want to close the channel twice
            return

        something_sent = self.send_some()
        if not something_sent:
            if not self._transmit_ready_event_done:
                self._transmit_ready_event_done = True
                self.broadcast_event(self.transmit_ready_event_name)

    def send_some(self):
        try:
            self._flush_some()
        except socket.error:
            logging.exception('Socket error')
            self.will_close = True
        except:
            self.logger.exception('Unexpected exception when flushing')
            self.will_close = True

        if self.close_when_flushed and not self.any_outbuf_has_data():
            self.close_when_flushed = False
            self.will_close = True

        if self.will_close:
            self.handle_close()

    def _flush_some(self):
        # Send as much data as possible to our client
        sent = 0
        dobreak = False

        while True:
            outbuf = self.outbufs[0]
            outbuflen = len(outbuf)
            if outbuflen <= 0:
                # self.outbufs[-1] must always be a writable outbuf
                if len(self.outbufs) > 1:
                    toclose = self.outbufs.pop(0)
                    try:
                        toclose._close()
                    except:
                        self.logger.exception(
                            'Unexpected error when closing an outbuf')
                    continue  # pragma: no cover (coverage bug, it is hit)
                else:
                    dobreak = True

            while outbuflen > 0:
                chunk = outbuf.get(self.send_bytes)
                if self.trace_dump:
                    self.logger and self.logger.info(
                        "Send data '%s' (%d bytes)\n%s" %
                        (self.name, len(chunk), hexdump('=>', chunk)))
                num_sent = self.send(chunk)
                if num_sent:
                    outbuf.skip(num_sent, True)
                    outbuflen -= num_sent
                    sent += num_sent
                else:
                    dobreak = True
                    break

            if dobreak:
                break

        if sent:
            self.last_activity = time.time()
            return True

        return False


# ---------------------------------------------------------------------------
# used for debugging.
# ---------------------------------------------------------------------------
def compact_traceback():
    t, v, tb = sys.exc_info()
    tbinfo = []
    if not tb:  # Must have a traceback
        raise AssertionError("traceback does not exist")
    while tb:
        tbinfo.append((
            tb.tb_frame.f_code.co_filename,
            tb.tb_frame.f_code.co_name,
            str(tb.tb_lineno)
            ))
        tb = tb.tb_next

    # just to be safe
    del tb

    file, function, line = tbinfo[-1]
    info = ' '.join(['[%s|%s|%s]' % x for x in tbinfo])
    return (file, function, line), t, v, info


def close_all_sockets(socket_map, ignore_all=False):
    for x in list(socket_map.values()):
        try:
            x.close()
        except OSError as x:
            if x.args[0] == EBADF:
                pass
            elif not ignore_all:
                raise
        except _reraised_exceptions:
            raise
        except:
            if not ignore_all:
                raise
    socket_map.clear()


#================================================================
#   Poll implementations
#================================================================
class _EPoll(object):
    """An epoll-based event loop using our C module for Python 2.5 systems"""
    _EPOLL_CTL_ADD = 1
    _EPOLL_CTL_DEL = 2
    _EPOLL_CTL_MOD = 3

    def __init__(self):
        self._epoll_fd = epoll.epoll_create()

    def fileno(self):
        return self._epoll_fd

    def close(self):
        os.close(self._epoll_fd)

    def register(self, fd, events):
        epoll.epoll_ctl(self._epoll_fd, self._EPOLL_CTL_ADD, fd, events)

    def modify(self, fd, events):
        epoll.epoll_ctl(self._epoll_fd, self._EPOLL_CTL_MOD, fd, events)

    def unregister(self, fd):
        epoll.epoll_ctl(self._epoll_fd, self._EPOLL_CTL_DEL, fd, 0)

    def poll(self, timeout):
        return epoll.epoll_wait(self._epoll_fd, int(timeout * 1000))


class _KQueue(object):
    """A kqueue-based event loop for BSD/Mac systems."""
    def __init__(self):
        self._kqueue = select.kqueue()
        self._active = {}

    def fileno(self):
        return self._kqueue.fileno()

    def close(self):
        self._kqueue.close()

    def register(self, fd, events):
        self._control(fd, events, select.KQ_EV_ADD)
        self._active[fd] = events

    def modify(self, fd, events):
        self.unregister(fd)
        self.register(fd, events)

    def unregister(self, fd):
        events = self._active.pop(fd)
        self._control(fd, events, select.KQ_EV_DELETE)

    def _control(self, fd, events, flags):
        kevents = []
        if events & IOLoop.WRITE:
            kevents.append(select.kevent(
                    fd, filter=select.KQ_FILTER_WRITE, flags=flags))
        if events & IOLoop.READ or not kevents:
            # Always read when there is not a write
            kevents.append(select.kevent(
                    fd, filter=select.KQ_FILTER_READ, flags=flags))
        # Even though control() takes a list, it seems to return EINVAL
        # on Mac OS X (10.6) when there is more than one event in the list.
        for kevent in kevents:
            self._kqueue.control([kevent], 0)

    def poll(self, timeout):
        kevents = self._kqueue.control(None, 1000, timeout)
        events = {}
        for kevent in kevents:
            fd = kevent.ident
            if kevent.filter == select.KQ_FILTER_READ:
                events[fd] = events.get(fd, 0) | IOLoop.READ
            if kevent.filter == select.KQ_FILTER_WRITE:
                if kevent.flags & select.KQ_EV_EOF:
                    # If an asynchronous connection is refused, kqueue
                    # returns a write event with the EOF flag set.
                    # Turn this into an error for consistency with the
                    # other IOLoop implementations.
                    # Note that for read events, EOF may be returned before
                    # all data has been consumed from the socket xbuffer,
                    # so we only check for EOF on write events.
                    events[fd] = IOLoop.ERROR
                else:
                    events[fd] = events.get(fd, 0) | IOLoop.WRITE
            if kevent.flags & select.KQ_EV_ERROR:
                events[fd] = events.get(fd, 0) | IOLoop.ERROR
        return events.items()


class _Select(object):
    """A simple, select()-based IOLoop implementation for non-Linux systems"""
    def __init__(self):
        self.read_fds = set()
        self.write_fds = set()
        self.error_fds = set()
        self.fd_sets = (self.read_fds, self.write_fds, self.error_fds)

    def close(self):
        pass

    def register(self, fd, events):
        if events & IOLoop.READ:
            self.read_fds.add(fd)
        if events & IOLoop.WRITE:
            self.write_fds.add(fd)
        if events & IOLoop.ERROR:
            self.error_fds.add(fd)
            # Closed connections are reported as errors by epoll and kqueue,
            # but as zero-byte reads by select, so when errors are requested
            # we need to listen for both read and error.
            self.read_fds.add(fd)

    def modify(self, fd, events):
        self.unregister(fd)
        self.register(fd, events)

    def unregister(self, fd):
        self.read_fds.discard(fd)
        self.write_fds.discard(fd)
        self.error_fds.discard(fd)

    def poll(self, timeout):
        readable, writeable, errors = select.select(
            self.read_fds, self.write_fds, self.error_fds, timeout)
        events = {}
        for fd in readable:
            events[fd] = events.get(fd, 0) | IOLoop.READ
        for fd in writeable:
            events[fd] = events.get(fd, 0) | IOLoop.WRITE
        for fd in errors:
            events[fd] = events.get(fd, 0) | IOLoop.ERROR
        return events.items()

#================================================================
#   Choose a poll implementation. Use epoll if it is available,
#   fall back to select() for non-Linux platforms
#================================================================
if hasattr(select, "epoll"):
    # Python 2.6+ on Linux
    _poll = select.epoll
elif hasattr(select, "kqueue"):
    # Python 2.6+ on BSD or Mac
    _poll = _KQueue
else:
    try:
        # Linux systems with our C module installed
        import epoll
        _poll = _EPoll
    except Exception:
        # All other systems
        if "linux" in sys.platform:
            logging.warning("epoll module not found; using select()")
        _poll = _Select

#_poll = _Select



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

import logging
import socket
import sys
import time
from ginsfsm.deferred import DeferredInterrupt

from ginsfsm.buffers import ReadOnlyFileBasedBuffer

from ginsfsm.compat import (
    tobytes,
    reraise,
    )

from ginsfsm.wsgi.utilities import (
    build_http_date,
    )

rename_headers = {
    'CONTENT_LENGTH' : 'CONTENT_LENGTH',
    'CONTENT_TYPE'   : 'CONTENT_TYPE',
    'CONNECTION'     : 'CONNECTION_TYPE',
    }

hop_by_hop = frozenset((
    'connection',
    'keep-alive',
    'proxy-authenticate',
    'proxy-authorization',
    'te',
    'trailers',
    'transfer-encoding',
    'upgrade'
    ))


class JustTesting(Exception):
    pass


class Task(object):
    close_on_finish = False
    status = '200 OK'
    wrote_header = False
    start_time = 0
    content_length = None
    content_bytes_written = 0
    logged_write_excess = False
    complete = False
    chunked_response = False

    def __init__(self, wsgi_server, gsock, request, ext_event=None):
        self.wsgi_server = wsgi_server
        self.gsock = gsock
        self.request = request
        self.ext_event = ext_event
        self.response_headers = []
        version = request.version
        if version not in ('1.0', '1.1'):
            # fall back to a version we support.
            version = '1.0'
        self.version = version

    def service(self):
        try:
            try:
                self.start()
                self.execute()
                self.finish()
            except socket.error:
                self.close_on_finish = True
                raise  # ??? if self.gsock.adj.log_socket_errors:
        except DeferredInterrupt:
            raise
        finally:
            pass

    def build_response_header(self):
        version = self.version
        # Figure out whether the connection should be closed.
        connection = self.request.headers.get('CONNECTION', '').lower()
        response_headers = self.response_headers
        content_length_header = None
        date_header = None
        server_header = None
        connection_close_header = None

        for i, (headername, headerval) in enumerate(response_headers):
            headername = '-'.join(
                [x.capitalize() for x in headername.split('-')]
                )
            if headername == 'Content-Length':
                content_length_header = headerval
            if headername == 'Date':
                date_header = headerval
            if headername == 'Server':
                server_header = headerval
            if headername == 'Connection':
                connection_close_header = headerval.lower()
            # replace with properly capitalized version
            response_headers[i] = (headername, headerval)

        if content_length_header is None and self.content_length is not None:
            content_length_header = str(self.content_length)
            self.response_headers.append(
                ('Content-Length', content_length_header)
                )

        def close_on_finish():
            if connection_close_header is None:
                response_headers.append(('Connection', 'close'))
            self.close_on_finish = True

        if version == '1.0':
            if connection == 'keep-alive':
                if not content_length_header:
                    close_on_finish()
                else:
                    response_headers.append(('Connection', 'Keep-Alive'))
            else:
                close_on_finish()

        elif version == '1.1':
            if connection == 'close':
                close_on_finish()

            if not content_length_header:
                response_headers.append(('Transfer-Encoding', 'chunked'))
                self.chunked_response = True
                if not self.close_on_finish:
                    close_on_finish()

            # under HTTP 1.1 keep-alive is default, no need to set the header
        else:
            raise AssertionError('neither HTTP/1.0 or HTTP/1.1')

        # Set the Server and Date field, if not yet specified. This is needed
        # if the server is used as a proxy.
        ident = self.wsgi_server.identity
        if not server_header:
            response_headers.append(('Server', ident))
        else:
            response_headers.append(('Via', ident))
        if not date_header:
            response_headers.append(('Date', build_http_date(self.start_time)))

        first_line = 'HTTP/%s %s' % (self.version, self.status)
        next_lines = ['%s: %s' % hv for hv in sorted(self.response_headers)]
        lines = [first_line] + next_lines
        res = '%s\r\n\r\n' % '\r\n'.join(lines)
        return tobytes(res)

    def remove_content_length_header(self):
        for i, (header_name, header_value) in enumerate(self.response_headers):
            if header_name.lower() == 'content-length':
                del self.response_headers[i]

    def start(self):
        self.start_time = time.time()

    def finish(self):
        if not self.wrote_header:
            self.write(b'')
        if self.chunked_response:
            # not self.write, it will chunk it!
            self.gsock.write_soon(b'0\r\n\r\n')
        self.gsock.send_some()

    def write(self, data):
        if not self.complete:
            raise RuntimeError('start_response was not called before body '
                               'written')
        gsock = self.gsock
        if not self.wrote_header:
            rh = self.build_response_header()
            gsock.write_soon(rh)
            self.wrote_header = True
        if data:
            towrite = data
            cl = self.content_length
            if self.chunked_response:
                # use chunked encoding response
                towrite = tobytes(hex(len(data))[2:].upper()) + b'\r\n'
                towrite += data + b'\r\n'
            elif cl is not None:
                towrite = data[:cl - self.content_bytes_written]
                self.content_bytes_written += len(towrite)
                if towrite != data and not self.logged_write_excess:
                    logging.error(
                        'application-written content exceeded the number of '
                        'bytes specified by Content-Length header (%s)' % cl)
                    self.logged_write_excess = True
            if towrite:
                gsock.write_soon(towrite)


class ErrorTask(Task):
    """ An error task produces an error response """
    complete = True

    def execute(self):
        e = self.request.error
        body = '%s\r\n\r\n%s' % (e.reason, e.body)
        tag = '\r\n\r\n(generated by ginsfsm.wsgi)'
        body = body + tag
        self.status = '%s %s' % (e.code, e.reason)
        cl = len(body)
        self.content_length = cl
        self.response_headers.append(('Content-Length', str(cl)))
        self.response_headers.append(('Content-Type', 'text/plain'))
        self.response_headers.append(('Connection', 'close'))
        self.close_on_finish = True
        self.write(tobytes(body))


class WSGITask(Task):
    """A WSGI task produces a response from a WSGI application.
    """
    environ = None

    def execute(self):
        env = self.get_environment()

        def start_response(status, headers, exc_info=None):
            if self.complete and not exc_info:
                raise AssertionError("start_response called a second time "
                                     "without providing exc_info.")
            if exc_info:
                try:
                    if self.complete:
                        # higher levels will catch and handle raised exception:
                        # 1. "service" method in task.py
                        # 2. "service" method in gsock.py
                        # 3. "handler_thread" method in task.py
                        reraise(exc_info[0], exc_info[1], exc_info[2])
                    else:
                        # As per WSGI spec existing headers must be cleared
                        self.response_headers = []
                finally:
                    exc_info = None

            self.complete = True

            if not status.__class__ is str:
                raise AssertionError('status %s is not a string' % status)

            self.status = status

            # Prepare the headers for output
            for k, v in headers:
                if not k.__class__ is str:
                    raise AssertionError(
                        'Header name %r is not a string in %r' % (k, (k, v))
                        )
                if not v.__class__ is str:
                    raise AssertionError(
                        'Header value %r is not a string in %r' % (v, (k, v))
                        )
                kl = k.lower()
                if kl == 'content-length':
                    self.content_length = int(v)
                elif kl in hop_by_hop:
                    raise AssertionError(
                        '%s is a "hop-by-hop" header; it cannot be used by '
                        'a WSGI application (see PEP 3333)' % k)

            self.response_headers.extend(headers)

            # Return a method used to write the response data.
            return self.write

        # Call the application to handle the request and write a response
        # TODO: do a multi wsgi-app
        try:
            app_iter = self.wsgi_server.application(env, start_response)
        except DeferredInterrupt:
            """ A gobj inside his gaplic need
                to wait for data from another gaplic.
            """
            raise

        try:
            if app_iter.__class__ is ReadOnlyFileBasedBuffer:
                cl = self.content_length
                size = app_iter.prepare(cl)
                if size:
                    if cl != size:
                        if cl is not None:
                            self.remove_content_length_header()
                        self.content_length = size
                    self.write(b'')  # generate headers
                    self.gsock.write_soon(app_iter)
                    return

            first_chunk_len = None
            for chunk in app_iter:
                if first_chunk_len is None:
                    first_chunk_len = len(chunk)
                    # Set a Content-Length header if one is not supplied.
                    # start_response may not have been called until first
                    # iteration as per PEP, so we must reinterrogate
                    # self.content_length here
                    if self.content_length is None:
                        app_iter_len = None
                        if hasattr(app_iter, '__len__'):
                            app_iter_len = len(app_iter)
                        if app_iter_len == 1:
                            self.content_length = first_chunk_len
                # transmit headers only after first iteration of the iterable
                # that returns a non-empty bytestring (PEP 3333)
                if chunk:
                    self.write(chunk)

            cl = self.content_length
            if cl is not None:
                if self.content_bytes_written != cl:
                    # close the connection so the client isn't sitting around
                    # waiting for more data when there are too few bytes
                    # to service content-length
                    self.close_on_finish = True
                    logging.error(
                        'application returned too few bytes (%s) '
                        'for specified Content-Length (%s) via app_iter' % (
                            self.content_bytes_written, cl),
                        )
        finally:
            if hasattr(app_iter, 'close'):
                app_iter.close()

    def get_environment(self):
        """Returns a WSGI environment."""
        environ = self.environ
        if environ is not None:
            # Return the cached copy.
            return environ

        request = self.request
        path = request.path
        gsock = self.gsock
        wsgi_server = self.wsgi_server

        while path and path.startswith('/'):
            path = path[1:]

        environ = {}
        environ['gsock'] = gsock
        environ['ext_event'] = self.ext_event
        environ['REQUEST_METHOD'] = request.command.upper()
        environ['SERVER_PORT'] = str(wsgi_server.effective_port)
        environ['SERVER_NAME'] = wsgi_server.server_name
        environ['SERVER_SOFTWARE'] = wsgi_server.identity
        environ['SERVER_PROTOCOL'] = 'HTTP/%s' % self.version
        environ['SCRIPT_NAME'] = ''
        environ['PATH_INFO'] = '/' + path
        environ['QUERY_STRING'] = request.query
        environ['REMOTE_ADDR'] = gsock.addr[0]

        for key, value in request.headers.items():
            value = value.strip()
            mykey = rename_headers.get(key, None)
            if mykey is None:
                mykey = 'HTTP_%s' % key
            if not mykey in environ:
                environ[mykey] = value

        # the following environment variables are required by the WSGI spec
        environ['wsgi.version'] = (1, 0)
        environ['wsgi.url_scheme'] = request.url_scheme
        environ['wsgi.errors'] = sys.stderr  # apps should use the logging module
        environ['wsgi.multithread'] = False
        environ['wsgi.multiprocess'] = False
        environ['wsgi.run_once'] = False
        environ['wsgi.input'] = request.get_body_stream()
        environ['wsgi.file_wrapper'] = ReadOnlyFileBasedBuffer

        self.environ = environ
        return environ



# -*- encoding: utf-8 -*-
"""
GObj :class:`GServerSock`
=========================

GObj for manage server socket.

.. autoclass:: GServerSock
    :members:

"""
import logging
import socket
from ginsfsm.c_sock import (
    GSock,
    GSOCK_FSM,  # used in documentation
    GSOCK_GCONFIG,  # used in documentation
)


class GServerSock(GSock):
    """  Server socket gobj.
    The incoming connections will create a new :class:`ginsfsm.c_sock.GSock`
    :term:`gobj`,
    that will be child of the :attr:`subscriber` :term:`gobj`.

        .. warning::  Remember destroy the accepted `gobj`
           with :func:`destroy_gobj` when the `gobj` has been disconnected.

           The `subcriber` knows when a new `gobj` has been accepted because it
           receives the ``'EV_CONNECTED'`` event.

           When the `subcriber` receives a ``'EV_DISCONNECTED'`` event must
           destroy the `gobj` because the connection ceases to exist.

    .. ginsfsm::
       :fsm: GSOCK_FSM
       :gconfig: GSOCK_GCONFIG

    *Input-Events:*
        The relationship is directly between the
        accepted :class:`ginsfsm.c_sock.GSock` gobj and the :attr:`subscriber`.

        See :class:`ginsfsm.c_sock.GSock` `input-events`.

    *Output-Events:*
        The relationship is directly between the
        accepted :class:`ginsfsm.c_sock.GSock` gobj and the :attr:`subscriber`.

        See :class:`ginsfsm.c_sock.GSock` `output-events`.
    """

    def __init__(self):
        GSock.__init__(self)
        self._n_clisrv = 0

    def start_up(self):
        """ Initialization zone.

        Start listen to (host,port).
        """
        if self.subscriber is None:
            self.subscriber = self.parent
        self.do_listen()

    def do_listen(self):
        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
        self.set_reuse_addr()
        self.bind((self.host, self.port))
        logging.info("Listening '%s' host '%s', port %d..." %
                 (self.name, self.host, self.port))
        self.listen(1024)

    def handle_accepted(self, sock, addr):
        # The socket options to set on receiving a connection.  It is a list of
        # (level, optname, value) tuples.  TCP_NODELAY disables the Nagle
        # algorithm for writes (Waitress already buffers its writes).
        # TODO: check origins for permitted source ip.
        socket_options = [
            (socket.SOL_TCP, socket.TCP_NODELAY, 1),
            ]
        for (level, optname, value) in socket_options:
            sock.setsockopt(level, optname, value)

        self._n_clisrv += 1
        if self.name and len(self.name):
            prefix_name = self.name
        else:
            prefix_name = None
        channel = '.clisrv-gsock-%x' % self._n_clisrv
        clisrv = self.create_gobj(
            prefix_name + channel if prefix_name else None,
            GSock,
            self.subscriber,
        )
        clisrv.subscribe_event(None, self.subscriber)
        clisrv.set_clisrv_socket(sock)
        clisrv.handle_connect()



import unittest

from ginsfsm.gaplic import (
    GAplic,
    setup_gaplic_process,
    setup_gaplic_thread,
    )
from ginsfsm.gobj import GObj
from ginsfsm.c_timer import GTimer
from ginsfsm.globals import (
    get_global_main_gaplic,
    set_global_main_gaplic,
    get_global_threads,
    get_global_subprocesses,
    shutdown,
    )

import logging
logging.basicConfig(level=logging.DEBUG)


def ac_timeout(self, event):
    self.counter += 1
    if self.counter < 10:
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=1)


FSM = {
    'event_list': ('EV_TIMEOUT',),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_TIMEOUT',    ac_timeout,           None),
        ),
    }
}


class GPrincipal(GObj):
    """  Principal gobj.
    """
    def __init__(self):
        GObj.__init__(self, FSM)

    def start_up(self):
        """ Create child and start the timer.
        """
        self.timer = self.create_gobj(
            None,       # unnamed gobj
            GTimer,     # gclass
            self        # parent
        )
        settings = {
            'GObj.trace_mach': True,
            'GObj.logger': logging,
        }
        self.overwrite_parameters(-1, **settings)
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=1)
        self.counter = 0


class MyGAplic(GAplic):
    counter = 0

    def mt_subprocess(self):
        self.counter += 1
        if self.counter > 10:
            shutdown()


def setup():
    ga = GAplic('GAplic-Subprocess')
    ga.create_gobj('principal-subprocess', GPrincipal, None)
    worker = setup_gaplic_process(ga)
    worker.start()

    ga = GAplic('GAplic-Thread')
    ga.create_gobj('principal-thread', GPrincipal, None)
    worker = setup_gaplic_thread(ga)
    worker.start()

    ga = MyGAplic('GAplic-Main')
    ga.create_gobj('principal-main', GPrincipal, None)
    set_global_main_gaplic(ga)
    #ga.start()
    return ga


class TestGAplic(unittest.TestCase):
    def setUp(self):
        self.ga = setup()

    def test_named_subscribe(self):
        while self.ga._loop():
            pass
        shutdown()

if __name__ == "__main__":
    setup()



"""
GObj :class:`OnClient`
======================

Stress with many connections and many data.

To run against :mod:`ginsfsm.examples.stress_server_echo`

The server echo the data.

It uses :class:`ginsfsm.c_connex.GConnex`.

.. autoclass:: OnClient
    :members:

"""

import datetime

import logging
logging.basicConfig(level=logging.DEBUG)

from ginsfsm.gobj import GObj
from ginsfsm.gaplic import GAplic
from ginsfsm.c_timer import GTimer
from ginsfsm.c_connex import GConnex

#===============================================================
#                   Client
#===============================================================
query = "GET / HTTP/1.1\r\n" + \
    "Host: \r\n" + \
    "\r\n"


def ac_client_timeout(self, event):
    if self.connex is None:
        self.connex = range(self.connections)

    if self.n_clients < self.connections:
        for i in range(self.n_clients, self.connections):
            self.n_clients += 1
            self.connex[i] = self.create_gobj(
                'client-%d' % i,
                GConnex,  # GSock,
                self,
                destinations=[(self.host, self.port)],
                transmit_ready_event_name=None,
            )

            self.connex[i].idx = i
            self.connex[i].conectado = 0
            self.connex[i].sended_msgs = 0
            self.connex[i].received_msgs = 0
            #self.connex[i].mt_connect(host='172.21.228.211', port=8084)
            #ret = self.connex[i].mt_connect(host='127.0.0.1', port=8000) #8082
            #if not ret:
            #    break
    print "conectados: %d" % self.n_connected_clients
    if self.n_connected_clients == self.connections:
        n_echoes = 0
        n_total = 0
        diff = datetime.timedelta(seconds=0)
        for cli in self.connex:
            #print 'cli %d txed_msgs %d, rxed_msgs %d' % (cli.idx, cli._gsock.txed_msgs, cli._gsock.rxed_msgs)
            if cli._gsock.rxed_msgs == cli._gsock.txed_msgs:
                n_total += 1
            if cli.sended_msgs == cli.received_msgs:
                n_echoes += 1
            diff += self.diff
        print "Echoes OK: %d of %d, taverage %s" % (n_echoes, self.connections, diff/self.connections)

        for cli in self.connex:
            if cli._gsock.connected:
                cli.sended_msgs = 1
                cli.received_msgs = 0
                cli.tx_time = datetime.datetime.now()
                self.send_event(cli, 'EV_TX_DATA', data=query) #data="HOLA")

    self.set_timeout(10)


def ac_client_connected(self, event):
    if not event.source[-1].conectado:
        self.n_connected_clients += 1
        event.source[-1].conectado = 1
    print "C: conectados: %d" % self.n_connected_clients


def ac_client_disconnected(self, event):
    if event.source[-1].conectado:
        self.n_connected_clients -= 1
        event.source[-1].conectado = 0
    print "D: conectados: %d" % self.n_connected_clients


def ac_client_rx_data(self, event):
    cli = self.connex[event.source[-1].idx]
    cli.received_msgs += 1
    cli.rx_time = datetime.datetime.now()
    diff = cli.rx_time - cli.tx_time
    if diff < self.min_response_time:
        self.min_response_time = diff
    if diff > self.max_response_time:
        self.max_response_time = diff
    self.diff = diff
    #print 'recibo:', event.data
    #print 'diff %s, min %s, max %s' % (diff, self.min_response_time, self.max_response_time)


CLIENT_FSM = {
    'event_list': (
        'EV_SET_TIMER: bottom output',
        'EV_TIMEOUT: bottom input',
        'EV_CONNECTED: bottom input',
        'EV_DISCONNECTED: bottom input',
        'EV_RX_DATA:bottom input',
    ),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_TIMEOUT',          ac_client_timeout,         'ST_IDLE'),
            ('EV_CONNECTED',        ac_client_connected,       'ST_IDLE'),
            ('EV_DISCONNECTED',     ac_client_disconnected,    'ST_IDLE'),
            ('EV_RX_DATA',          ac_client_rx_data,         'ST_IDLE'),
        ),
    }
}


CLIENT_GCONFIG = {  # type, default_value, flag, validate_function, desc
    'verbose': [int, 0, 0, None, "Increase output verbosity. Values [0,1,2]"],
    'connections': [int, 0, 0, None, "Limit of connections to be reached."],
    'host': [str, '127.0.0.1', 0, None, "Port."],
    'port': [int, 8000, 0, None, "Port."],
}


class OnClient(GObj):
    """  Client GObj.

    .. ginsfsm::
       :fsm: CLIENT_FSM
       :gconfig: CLIENT_GCONFIG

    *Input-Events:*

        * :attr:`'EV_TIMEOUT'`: Timer over.
            Start the machine.

        * :attr:`'EV_CONNECTED'`: New client.

        * :attr:`'EV_DISCONNECTED'`: Client disconnected.

        * :attr:`'EV_RX_DATA'`: Receiving data.

    *Output-Events:*

        * :attr:`'EV_START_TIMER'`: Start timer.

    """
    def __init__(self):
        GObj.__init__(self, CLIENT_FSM, CLIENT_GCONFIG)
        self.n_clients = 0
        self.n_connected_clients = 0

    def start_up(self):
        self.timer = self.create_gobj(
            None,
            GTimer,
            self
        )
        self.connex = None
        self.sended_msgs = 0
        self.received_msgs = 0
        self.min_response_time = datetime.timedelta(seconds=100)
        self.max_response_time = datetime.timedelta(seconds=0)
        self.diff = datetime.timedelta(seconds=0)

        if self.verbose > 1:
            settings = {
                'GObj.trace_mach': True,
                'GObj.logger': logging,
            }
            self.overwrite_parameters(-1, **settings)

        self.set_timeout(1)

    def set_timeout(self, seconds):
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=seconds)

    def clear_timeout(self):
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=-1)


#===============================================================
#                   Main
#===============================================================
if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "host",
        nargs='?', default='127.0.0.1',
        help="Host."
    )
    parser.add_argument(
        "port",
        type=int,
        nargs='?', default=8000,
        help="Port."
    )
    parser.add_argument(
        "connections",
        type=int,
        nargs='?', default=100,
        help="Limit of connections to be reached."
    )
    parser.add_argument(
        "--run-as-process",
        action="store_true",
        help="Run the server as subprocess. By default it runs as thread."
    )
    parser.add_argument(
        "-v",
        "--verbose",
        help="Increase output verbosity",
        type=int,
        choices=[0, 1, 2],
        default=0,
    )
    args = parser.parse_args()
    run_as_process = args.run_as_process

    ga_cli = GAplic('Client')
    ga_cli.create_gobj(
        'client',
        OnClient,
        None,
        host=args.host,
        port=args.port,
        connections=args.connections,
    )

    try:
        ga_cli.mt_process()
    except (KeyboardInterrupt, SystemExit):
        from ginsfsm.c_sock import close_all_sockets
        close_all_sockets(ga_cli._socket_map)
        print('Program stopped')



"""
GObj :class:`OnTimer`
=====================

Utility for executing a ``command`` cyclically each ``seconds`` seconds.

The timer is supplied by :class:`ginsfsm.c_timer.GTimer`.

Dependencies:

* Envoy: `Subprocesses for Humans by kennethreitz \
<https://github.com/kennethreitz/envoy>`_.

.. autoclass:: OnTimer
    :members:

"""
import envoy

from ginsfsm.gobj import GObj
from ginsfsm.gaplic import GAplic
from ginsfsm.c_timer import GTimer

import logging
logging.basicConfig(level=logging.DEBUG)


def ac_exec_command(self, event):
    if self.verbose:
        print('Executing %s...' % self.command)

    response = envoy.run(self.command)

    if self.verbose:
        print(response.std_out)

    self.send_event(self.timer, 'EV_SET_TIMER', seconds=self.seconds)


ONTIMER_FSM = {
    'event_list': (
        'EV_SET_TIMER: bottom output',
        'EV_TIMEOUT: bottom input',
    ),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_TIMEOUT', ac_exec_command, 'ST_IDLE'),
        ),
    }
}

ONTIMER_GCONFIG = {  # type, default_value, flag, validate_function, desc
    'seconds': [int, 2, 0, None, "Seconds to repeat the command."],
    'verbose': [int, 0, 0, None, "Increase output verbosity. Values [0,1,2]"],
    'command': [str, 'ls', 0, None, "Command to execute."],
}


class OnTimer(GObj):
    """  OnTimer GObj.

    .. ginsfsm::
       :fsm: ONTIMER_FSM
       :gconfig: ONTIMER_GCONFIG

    *Input-Events:*

        * :attr:`'EV_TIMEOUT'`: Timer over.
            Execute the ``command``.

    *Output-Events:*

        * :attr:`'EV_START_TIMER'`: Start timer.

    """
    def __init__(self):
        GObj.__init__(self, ONTIMER_FSM, ONTIMER_GCONFIG)

    def start_up(self):
        """
        """
        self.timer = self.create_gobj(
            None,       # unnamed gobj
            GTimer,     # gclass
            self        # parent
        )

        if self.verbose > 1:
            settings = {
                'GObj.trace_mach': True,
                'GObj.logger': logging,
            }
            self.overwrite_parameters(-1, **settings)

        self.send_event(self.timer, 'EV_SET_TIMER', seconds=self.seconds)


if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "seconds",
        type=int,
        nargs='?', default=2,
        help="Seconds to repeat the command."
    )
    parser.add_argument(
        "command",
        nargs=argparse.REMAINDER,
        default='ls',
        help="Command to execute."
    )
    parser.add_argument(
        "-v",
        "--verbose",
        help="Increase output verbosity",
        type=int,
        choices=[0, 1, 2],
        default=2,
    )
    args = parser.parse_args()

    ga = GAplic()
    ga.create_gobj(
        'ontimer',
        OnTimer,
        None,
        verbose=args.verbose,
        seconds=args.seconds,
        command=''.join(args.command) or 'ls')
    try:
        ga.mt_process()
    except (KeyboardInterrupt, SystemExit):
        print('Program stopped')



# -*- encoding: utf-8 -*-
""" Container of gobjs.


.. autoclass:: GAplic
    :members: create_gobj, destroy_gobj,
        register_unique_gobj, deregister_unique_gobj, search_unique_gobj,
        mt_process, mt_subprocess

.. autofunction:: setup_gaplic_thread

.. autoclass:: GAplicThreadWorker
    :members:

.. autofunction:: setup_gaplic_process

.. autoclass:: GAplicProcessWorker
    :members:

"""
import time
import threading
import logging
from collections import deque

from ginsfsm.globals import (
    set_global_main_gaplic,
    set_global_app,
    add_global_thread,
    add_global_subprocess,
    get_gaplic_by_thread_ident,
    )
from ginsfsm.compat import (
    string_types,
    iterkeys_,
    )
from ginsfsm.deferred import (
    DeferredList,
    )
from ginsfsm.c_sock import (
    poll_loop,
    close_all_sockets,
    _poll,
    GSock,
    )
from ginsfsm.gobj import (
    GObj,
    )


def _start_timer(seconds):
    """ Start a timer of :param:`seconds` seconds.
    The returned value must be used to check the end of the timer
    with _test_timer() function.
    """
    timer = time.time()
    timer = timer + seconds
    return timer


def _test_timer(value):
    """ Check if timer :param:`value` has ended.
    Return True if the timer has elapsed, False otherwise.
    WARNING: it will fail when system clock has been changed.
    TODO: check if system clock has been changed.
    """
    timer_actual = time.time()
    if timer_actual >= value:
        return True
    else:
        return False


class _XTimer(object):
    """  Group attributes for timing.
    :param:`got_timer` callback will be executed :param:`sec` seconds.
    The callback will be called with :param:`param1` parameter.
    If :param:`autostart` is True, the timer will be cyclic.
    """
    def __init__(self, sec, got_timer_func, param1, autostart):
        self.sec = sec
        self.got_timer_func = got_timer_func
        self.param1 = param1
        self.autostart = autostart


def ac_deferred_callback(self, event):
    deferred_ref = event.deferred_ref
    self.deferred_list(deferred_ref, ext_event=event)
    self.deferred_list.delete(deferred_ref)

GAPLIC_FSM = {
    'event_list': ('EV_DEFERRED_CALLBACK',),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_DEFERRED_CALLBACK', ac_deferred_callback, 'ST_IDLE'),
        ),
    }
}


class GAplic(GObj):
    """ Container of gobj's running under the same process or thread.

    :param name: name of the gaplic, default is ``None``.
    :param settings: keyword arguments, with the global parameters from a
        ini configfile.

        .. note::

           The parameters can be dot named to include
           the :term:`named-gobj`'s destination of the parameters.

        .. warning::

           The value of these global parameters **overriden**
           the values arguments passed in :meth:`create_gobj` method.

    GAplic is the main boss.
    Manage the timer's, event queues, etc.
    Supplies register, deregister and search or named-events.

    Example::

        if __name__ == "__main__":
            ga = GAplic(name='Example1')
            ga.create_gobj('test_aplic', GPrincipal, None)
            try:
                ga.mt_process()
            except KeyboardInterrupt:
                print('Program stopped')

    """
    def __init__(self, name=None, **settings):
        GObj.__init__(self, GAPLIC_FSM)
        self.name = name
        self.global_settings = settings.copy()
        self.do_exit = multiprocessing.Event()  # Call shutdown() to stop gaplic
        """threading.Event() or multiprocessing.Event() object
        to signal the shutdown of gaplic."""
        self.loop_timeout = 0.5     # timeout to select(),poll() function.
        """Loop timeout. Default 0.5 seconds."""
        self._impl_poll = _poll()   # Used by gsock. epoll() implementation.
        self._socket_map = {}       # Used by gsock. Dict {fd:Gobj}
        self._gotter_timers = {}    # Dict with timers  {_XTimer:timer value}
        self._qevent = deque()      # queue for post events.
        self._inside = 0            # to tab machine trace.
        self._unique_named_gobjs = {}
        self.thread_ident = 0
        self.thread_name = 0
        self.gaplic = self
        self.deferred_list = DeferredList()

        logger = settings.get('logger', None)
        if isinstance(logger, string_types):
            self.logger = logging.getLogger(logger)
        if self.logger is None:
            self.logger = logging.getLogger(__name__)  # TODO use package caller
        self.logger and self.logger.info('GAplic (%s) initiated' % self.name)

    def _increase_inside(self):
        self._inside += 1

    def _decrease_inside(self):
        self._inside -= 1

    def _tab(self):
        if self._inside <= 0:
            spaces = 1
        else:
            spaces = self._inside*2
        pad = ' ' * spaces
        return pad

    def create_gobj(self, name, gclass, parent, **kw):
        """ Factory function to create gobj's instances.

        :param name: Name of gobj.
            You can create :term:`named-gobj` or :term:`unnamed-gobj` gobjs.
            :term:`named-gobj` gobjs can be reached by his name,
            but the :term:`unnamed-gobj` gobjs
            are only knowing by their `pointer`.

        :param gclass: `gclass` is the GObj type used to create the
            new gobj.It's must be a derived class of :class:`ginsfsm.gobj.GObj`.
        :param parent: parent of the new :term:`gobj`.
            If `None`, the gobj will be a :term:`principal` gobj and
            their parent will be :term:`gaplic`.
        :param kw: Attributes that are added to the new :term:`gobj`.
            All the keyword arguments used in the function
            **are added as attributes** to the created :term:`gobj`.
            You must consult the attributes supported
            by each `gclass` type.
        :rtype: new gobj instance.

        When a :term:`gobj` is created by the factory function, it's added to
        their parent child list :attr:`ginsfsm.gobj.GObj.dl_childs`,
        and several attributes are created:

        * **parent**: the parent :term:`gobj` of the created :term:`gobj`.
        * **gaplic**: the :term:`gaplic` of the created :term:`gobj`.

        If the `gclass` is subclass of :class:`ginsfsm.c_sock.GSock`
        two private attributes are added to the created  :term:`gobj`:

        * **_socket_map**: dictionary of open sockets.
        * **_impl_poll**: poll implementation: can be epoll, select, KQueue..
          the best found option.

        It's the base of the asynchronous behavior.
        """
        if parent is None:
            parent = self

        _gaplic_kw = {
            'gaplic': self,
            'logger': self.logger,
            'create_gobj': self.create_gobj,
            'enqueue_event': self.enqueue_event,
            'register_unique_gobj': self.register_unique_gobj,
            'deregister_unique_gobj': self.deregister_unique_gobj,
            'search_unique_gobj': self.search_unique_gobj,
            '_increase_inside': self._increase_inside,
            '_decrease_inside': self._decrease_inside,
            '_tab': self._tab,
        }

        if issubclass(gclass, GSock):
            _gaplic_kw.update({
                '_socket_map': self._socket_map,
                '_impl_poll': self._impl_poll
            })

        kw.update(_gaplic_kw)

        gobj = GObj.create_gobj(self, name, gclass, parent, **kw)
        return gobj

    def get_named_gobjs(self):
        """ Return the list of :term:`named-gobj`'s.
        """
        return [name for name in iterkeys_(self._unique_named_gobjs)]

    def register_unique_gobj(self, gobj):
        """ Register a :term:`named-gobj`.
        """
        named_gobj = self._unique_named_gobjs.get(gobj.name, None)
        if named_gobj is not None:
            return False
        self._unique_named_gobjs[gobj.name] = gobj
        return True

    def deregister_unique_gobj(self, gobj):
        """ Deregister a :term:`named-gobj`.
        """
        named_gobj = self._unique_named_gobjs.get(gobj.name, None)
        if named_gobj is not None:
            del self._unique_named_gobjs[gobj.name]
            return True
        return False

    def search_unique_gobj(self, gobj_name):
        """ Search a :term:`named-gobj`.
        """
        named_gobj = self._unique_named_gobjs.get(gobj_name, None)
        return named_gobj

    def _loop(self):
        """ process event queue, timer queue, and epoll.
        Return True if there is some remain event for be proccessed.
        Useful for testing purposes.
        """
        if not self.thread_ident:
            self.thread_ident = threading.current_thread().ident
            self.thread_name = threading.current_thread().name
        remain = self._process_qevent()
        if remain:
            timeout = 0.01
        else:
            timeout = self.loop_timeout
        some_event = poll_loop(self._socket_map, self._impl_poll, timeout)
        if some_event:
            remain = True
        remain |= self._process_timer()

        self.mt_subprocess()
        return remain

    def mt_process(self):
        self.start()

    def start(self):
        """ Infinite loop function to be called by thread or process.
        It's call ``start`` to be used by pasterdeploy
        """
        while True:
            # with do_exit Event set (being thread or process),
            # wait to event set to exit, ignoring KeyboardInterrupt.
            try:
                if self.do_exit.is_set():
                    close_all_sockets(self._socket_map)
                    break
                self._loop()
            except (KeyboardInterrupt, SystemExit):
                close_all_sockets(self._socket_map)
                raise

    def shutdown(self):
        """ Signalize the worker to shutdown """
        self.do_exit.set()

    def mt_subprocess(self):
        """ Subclass :class:`GAplic` class and override this function
        to do extra work in the infinite loop.
        """

    def enqueue_event(self, event):
        """ Post the event in the next :term:`gaplic` loop cycle,
        not right now.

        :param event: :term:`event` to send.
        :param destination: destination :term:`gobj` of the event.
        :param kw: keyword argument with data associated to event.

            .. note::
                All the keyword arguments **are added as attributes** to
                the sent :term:`event`.

        Same as :meth:`send_event` function but the event is sent in the
        next :term:`gaplic` loop cycle, not right now.

        It **does not return** the return of the executed action because the
        action it's executed later, in the next loop cycle.

        It's mandatory use this function, if the `destination`
        :term:`gobj` is not local.

        .. note:: It **DOES NOT** return the return of the executed action
            because the action it's executed later, in the next loop cycle,
            so you **CANNOT** receive valid direct data from the action.

        .. warning:: If you use :meth:`post_event` without a :term:`gaplic`
            then a :exc:`GAplicError` exception will be raised.

        ``destination`` must be a `string` or :class:`GObj` types, otherwise a
        :exc:`GObjError` will be raised.

        ``event`` must be a `string` or :class:`Event` types, otherwise a
        :exc:`EventError` will be raised.

        If ``event`` is an :class:`Event` instance, a new :class:`Event`
        duplicated instance is returned, but it will be updated with
        the new ``destination`` and ``kw`` keyword arguments.

        .. note::
            All the keyword arguments used in the factory function
            **are added as attributes** to the created :term:`event` instance.
            You must consult the attributes supported by each machine's event.
        """
        self._qevent.append(event)

    def _process_qevent(self):
        """ Return True if remains events.
        """
        # ln = len(self._qevent)
        # print "qevent...........%d" % (ln)
        it = 0
        maximum = 10
        while True:
            if it > maximum:
                # balance the work
                return True
            try:
                event = self._qevent.popleft()
            except IndexError:
                break
            else:
                it += 1
                #TODO consider names of another gaplics
                destination = self._resolv_destination(event.destination)
                cur_ident = threading.current_thread().ident
                dst_ident = destination.gaplic.thread_ident
                if cur_ident == dst_ident:
                    self.send_event(destination, event.event_name, **event.kw)
                else:
                    # Yeah, send to another gaplic
                    gaplic = get_gaplic_by_thread_ident(dst_ident)
                    if gaplic:
                        gaplic.enqueue_event(event)
        return False

    def _process_timer(self):
        # dont use iteritems() items(),
        # some xtimer can remove during processing timers
        some_event = False
        try:
            for xtimer in iterkeys_(self._gotter_timers):
                try:
                    value = self._gotter_timers[xtimer]
                except KeyError:
                    # timer deleted while loop.
                    continue
                some_event = True
                if value and _test_timer(value):
                    if xtimer.autostart:
                        self._gotter_timers[xtimer] = _start_timer(xtimer.sec)
                    else:
                        self._gotter_timers[xtimer] = 0
                    if xtimer.param1 is None:
                        xtimer.got_timer_func()
                    else:
                        xtimer.got_timer_func(xtimer.param1)
                    if not xtimer.autostart:
                        self._gotter_timers.pop(xtimer)
        except RuntimeError:
            # timer deleted while loop.
            some_event = True
        return some_event

    def _setTimeout(self, sec, got_timer_func, param1=None, autostart=False):
        """ Set a callback to be executed in ``sec`` seconds.
        Function used by :class:`GTimer` gobj. Not for general use.
        Return an object to be used in :func:`clearTimeout`.
        """
        xtimer = _XTimer(sec, got_timer_func, param1, autostart)
        self._gotter_timers[xtimer] = _start_timer(sec)
        return xtimer

    def _clearTimeout(self, xtimer):
        """ Clear callback timeout.
        Function used by :class:`GTimer` gobj. Not for general use.
        """
        t = self._gotter_timers.get(xtimer, None)
        if t:
            # prevent timer cleared in proces_timer loop
            self._gotter_timers[xtimer] = 0
            self._gotter_timers.pop(xtimer)

#===============================================================
#                   Thread wrapper for gaplic
#===============================================================
import threading


class GAplicThreadWorker(threading.Thread):
    """ Class derived from :class:`threading.Thread` to run gaplic."""
    def __init__(self, gaplic):
        threading.Thread.__init__(self)
        self.daemon = True
        self.gaplic = gaplic

    def run(self):
        self.gaplic.mt_process()

    def join(self, timeout=10.0):   # wait until 10 seconds for thread killed.
        """ Wait for worker to shutdown, until ``timeout`` seconds."""
        super(GAplicThreadWorker, self).join(timeout)


def setup_gaplic_thread(gaplic):
    """ Run gaplic as thread.
    Return the worker.
    You must call worker.start() to run the thread.
    """
    worker = GAplicThreadWorker(gaplic)
    add_global_thread(worker)
    return worker

#===============================================================
#                   Process wrapper for gaplic
#===============================================================
import multiprocessing


class GAplicProcessWorker(multiprocessing.Process):
    """ Class derived from :class:`multiprocessing.Process` to run gaplic."""
    def __init__(self, gaplic):
        multiprocessing.Process.__init__(self)
        self.daemon = True
        self.gaplic = gaplic

    def run(self):
        self.gaplic.mt_process()

    def join(self, timeout=10.0):   # wait until 10 seconds for process killed.
        """ Wait for worker to shutdown, until ``timeout`` seconds."""
        super(GAplicProcessWorker, self).join(timeout)


def setup_gaplic_process(gaplic):
    """ Run gaplic as process.
    Return the worker.
    You must call worker.start() to run the subprocess.
    """
    worker = GAplicProcessWorker(gaplic)
    add_global_subprocess(worker)
    return worker


def gaplic_factory(loader, global_config, main,
                   threads=None, subprocesses=None, others=None):
    """ To use with PasteDeploy in composite, example::

        [composite:main]
        use = call:ginsfsm.gaplic:gaplic_factory
        main = wsgi-server
        threads =
        subprocesses =
        others = wsgi-application

        [app:wsgi-server]
        use = call:ginsfsm.examples.example6-wsgi-server:main
        host = 0.0.0.0
        port = 8001
        wsgi-server.application = app:wsgi-application

        [app:wsgi-application]
        use = call:ginsfsm.examples.example6-wsgi-server:paste_app_factory

    The prototype for ``others`` (paste app factory) is::

        def paste_app_factory(global_config, **local_conf):
            return wsgi-application

    The prototype for ``main``, ``threads`` and ``subprocesses`` is:

        def main(global_config, **settings):
            return gaplic-instance

    """
    others = others.split()
    threads = threads.split()
    subprocesses = subprocesses.split()

    for other in others:
        app = loader.get_app(other)
        set_global_app(other, app)

    gaplic_main = loader.get_app(main)
    set_global_main_gaplic(gaplic_main)

    for thread in threads:
        gaplic = loader.get_app(thread)
        worker = GAplicThreadWorker(gaplic)
        worker.start()

    for subprocess in subprocesses:
        gaplic = loader.get_app(subprocess)
        worker = GAplicProcessWorker(gaplic)
        worker.start()

    return gaplic_main



"""
GObj :class:`OnServer` and :class:`OnClient`
============================================

Utility for check a server/client :term:`gaplic`'s
running as thread or subprocesses and stress the connections.

It uses :class:`ginsfsm.c_srv_sock.GServerSock`
and :class:`ginsfsm.c_sock.GSock`.

Run two gaplics. One is the server, the other the client.

Stress with many connections.

Configuration:
    * The server can run as thread o subprocess.
    * Limit of client connections to be reached.

.. autoclass:: OnServer
    :members:

.. autoclass:: OnClient
    :members:
"""

import time
import logging
logging.basicConfig(level=logging.DEBUG)

from ginsfsm.gobj import GObj
from ginsfsm.gaplic import GAplic
from ginsfsm.c_timer import GTimer
from ginsfsm.c_connex import GConnex
from ginsfsm.c_srv_sock import GServerSock


#===============================================================
#                   Server
#===============================================================

def ac_clisrv_timeout(self, event):
    self.set_timeout(5)
    pass


def ac_clisrv_connected(self, event):
    if len(self.dl_childs) == 3:
        self.start_time = time.clock()

    print('connected FROM %s' % str(event.peername))
    print("Server's clients: %d" % len(self.dl_childs))

    if len(self.dl_childs) == self.connections + 2:
        end_time = time.clock()
        print("Time for %d connections: %f seconds" % (
            self.connections,
            end_time - self.start_time))


def ac_clisrv_disconnected(self, event):
    self.destroy_gobj(event.source[-1])


SERVER_FSM = {
    'event_list': (
        'EV_SET_TIMER: bottom output',
        'EV_TIMEOUT: bottom input',
        'EV_CONNECTED: bottom input',
        'EV_DISCONNECTED: bottom input',
    ),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_TIMEOUT',          ac_clisrv_timeout,         None),
            ('EV_CONNECTED',        ac_clisrv_connected,       None),
            ('EV_DISCONNECTED',     ac_clisrv_disconnected,    None),
        ),
    }
}

SERVER_GCONFIG = {  # type, default_value, flag, validate_function, desc
    'verbose': [int, 0, 0, None, "Increase output verbosity. Values [0,1,2]"],
    'connections': [int, 0, 0, None, "Limit of connections to be reached."],
}


class OnServer(GObj):
    """  Server GObj.

    .. ginsfsm::
       :fsm: SERVER_FSM
       :gconfig: SERVER_GCONFIG

    *Input-Events:*

        * :attr:`'EV_TIMEOUT'`: Timer over.
            Start the machine.

        * :attr:`'EV_CONNECTED'`: New client.

        * :attr:`'EV_DISCONNECTED'`: Client disconnected.

    *Output-Events:*

        * :attr:`'EV_START_TIMER'`: Start timer.

    """
    def __init__(self):
        GObj.__init__(self, SERVER_FSM, SERVER_GCONFIG)
        self.start_time = 0

    def start_up(self):
        self.timer = self.create_gobj(
            None,
            GTimer,
            self)

        self.server = self.create_gobj(
            None,
            GServerSock,
            self,
            host='127.0.0.1',
            port=8000,
            # only want receive EV_CONNECTED/EV_DISCONNECTED event
            rx_data_event_name=None,
            transmit_ready_event_name=None,
        )

        if self.verbose > 1:
            settings = {
                'GObj.trace_mach': True,
                'GObj.logger': logging,
            }
            self.overwrite_parameters(-1, **settings)

        self.set_timeout(5)

    def set_timeout(self, seconds):
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=seconds)

    def clear_timeout(self):
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=-1)


#===============================================================
#                   Client
#===============================================================

def ac_client_timeout(self, event):
    if self.connex is None:
        self.connex = range(self.connections)
        for i in self.connex:
            print i
            self.connex[i] = self.create_gobj(
                'client-%02d' % i,
                GConnex,
                self,
                destinations=[('127.0.0.1', 8000)],
                # only want receive EV_CONNECTED/EV_DISCONNECTED event
                rx_data_event_name=None,
                transmit_ready_event_name=None,
            )
    self.set_timeout(10)


def ac_client_connected(self, event):
    pass


def ac_client_disconnected(self, event):
    pass


CLIENT_FSM = {
    'event_list': (
        'EV_SET_TIMER: bottom output',
        'EV_TIMEOUT: bottom input',
        'EV_CONNECTED: bottom input',
        'EV_DISCONNECTED: bottom input',
    ),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_TIMEOUT',          ac_client_timeout,         None),
            ('EV_CONNECTED',        ac_client_connected,       None),
            ('EV_DISCONNECTED',     ac_client_disconnected,    None),
        ),
    }
}

CLIENT_GCONFIG = {  # type, default_value, flag, validate_function, desc
    'verbose': [int, 0, 0, None, "Increase output verbosity. Values [0,1,2]"],
    'connections': [int, 0, 0, None, "Limit of connections to be reached."],
}


class OnClient(GObj):
    """  Client GObj.

    .. ginsfsm::
       :fsm: CLIENT_FSM
       :gconfig: CLIENT_GCONFIG

    *Input-Events:*

        * :attr:`'EV_TIMEOUT'`: Timer over.
            Start the machine.

        * :attr:`'EV_CONNECTED'`: Client connected.

        * :attr:`'EV_DISCONNECTED'`: Client disconnected.

    *Output-Events:*

        * :attr:`'EV_START_TIMER'`: Start timer.

    """
    def __init__(self):
        GObj.__init__(self, CLIENT_FSM, CLIENT_GCONFIG)

    def start_up(self):
        self.timer = self.create_gobj(
            None,
            GTimer,
            self,
        )

        if self.verbose > 1:
            settings = {
                'GObj.trace_mach': True,
                'GObj.logger': logging,
            }
            self.overwrite_parameters(-1, **settings)

        self.connex = None
        self.set_timeout(1)

    def set_timeout(self, seconds):
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=seconds)

    def clear_timeout(self):
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=-1)


#===============================================================
#                   Main
#===============================================================
from ginsfsm.gaplic import setup_gaplic_thread, setup_gaplic_process

if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "connections",
        type=int,
        nargs='?', default=400,
        help="Limit of connections to be reached."
    )
    parser.add_argument(
        "--run-as-process",
        action="store_true",
        help="Run the server as subprocess. By default it runs as thread."
    )
    parser.add_argument(
        "-v",
        "--verbose",
        help="Increase output verbosity",
        type=int,
        choices=[0, 1, 2],
        default=0,
    )
    args = parser.parse_args()
    run_as_process = args.run_as_process

    if run_as_process:
        # run server gaplic as child daemon process
        ga_srv = GAplic('Server')
        ga_srv.create_gobj(
            'server',
            OnServer,
            None,
            connections=args.connections,
            verbose=args.verbose,
        )
        srv_worker = setup_gaplic_process(ga_srv)
        srv_worker.start()

        # run client gaplic as main process
        ga_cli = GAplic('Client')
        ga_cli.create_gobj(
            'client',
            OnClient,
            None,
            connections=args.connections,
            verbose=args.verbose,
        )

        try:
            ga_cli.start()
        except (KeyboardInterrupt, SystemExit):
            ga_srv.shutdown()
            srv_worker.join()
            print('Program stopped')

    else:
        # run server gaplic as thread
        ga_srv = GAplic('Server')
        ga_srv.create_gobj(
            'server',
            OnServer,
            None,
            connections=args.connections,
            verbose=args.verbose,
        )
        srv_worker = setup_gaplic_thread(ga_srv)
        srv_worker.start()

        # run client gaplic as main process
        ga_cli = GAplic('Client')
        ga_cli.create_gobj(
            'client',
            OnClient,
            None,
            connections=args.connections,
            verbose=args.verbose,
        )

        try:
            ga_cli.start()
        except (KeyboardInterrupt, SystemExit):
            ga_srv.shutdown()
            srv_worker.join()
            print('Program stopped')



from collections import deque
import unittest

from ginsfsm.gaplic import GAplic
from ginsfsm.gobj import GObj, NotUniqueGObjError
from ginsfsm.c_timer import GTimer

import logging
logging.basicConfig(level=logging.DEBUG)


def ac_check_queue(self, event):
    tx_events = 0
    while len(self.dl_query):
        try:
            data = self.dl_query.popleft()
        except IndexError:
            break
        else:
            self.send_event(self, 'EV_QUERY', data=data)
            tx_events += 1
    return tx_events


def ac_send_query(self, event):
    data = event.data
    self.send_event(self.timer, 'EV_SET_TIMER', seconds=0)  # simulate timeout
    return data


def ac_timeout_wait_resp(self, event):
    self.set_new_state('ST_IDLE')
    self.send_event(self.timer, 'EV_SET_TIMER', seconds=0)  # simulate timeout


def ac_enqueue_query(self, event):
    data = event.data
    self.dl_query.append(data)


FSM = {
    'event_list': (
        'EV_QUERY:output',
        'EV_TIMEOUT',
    ),
    'state_list': ('ST_IDLE', 'ST_WAIT_RESP'),
    'machine': {
        'ST_IDLE':
        (
            ('EV_TIMEOUT',    ac_check_queue,           None),
            ('EV_QUERY',      ac_send_query,            'ST_WAIT_RESP'),
        ),

        'ST_WAIT_RESP':
        (
            ('EV_TIMEOUT',    ac_timeout_wait_resp,     None),
            ('EV_QUERY',      ac_enqueue_query,         None),
        ),
    }
}


class GPrincipal(GObj):
    """  Principal gobj.
    """
    def __init__(self):
        GObj.__init__(self, FSM)

    def start_up(self):
        """ Create child and start the timer.
        """
        self.timer = self.create_gobj(
            None,       # unnamed gobj
            GTimer,     # gclass
            self        # parent
        )
        settings = {
            'GObj.trace_mach': True,
            'GObj.logger': logging,
        }
        self.overwrite_parameters(-1, **settings)
        self.dl_query = deque()


class TestGAplic(unittest.TestCase):
    def setUp(self):
        self.gaplic = GAplic(name='TestGAplic')
        self.principal = self.gaplic.create_gobj('principal', GPrincipal, None)

    def tearDown(self):
        self.gaplic.destroy_gobj(self.principal)

    def test_create_gobj(self):
        self.assertRaises(NotUniqueGObjError,
            self.gaplic.create_gobj, 'principal', GPrincipal, None)

    def test_named_subscribe(self):
        self.principal.subscribe_event('EV_QUERY', 'principal')

    def test_send_event(self):
        data = 'XXX'
        state = self.principal.get_current_state()
        self.assertEqual(state, 'ST_IDLE')
        ret = self.principal.send_event('principal', 'EV_QUERY', data=data)
        self.assertEqual(ret, data)
        state = self.principal.get_current_state()
        self.assertEqual(state, 'ST_WAIT_RESP')
        ret = self.principal.send_event(self.principal, 'EV_QUERY', data=data)
        self.assertEqual(ret, None)
        self.assertEqual(len(self.principal.dl_query), 1)
        #self.gaplic.mt_process()
        self.gaplic._loop()
        self.gaplic._loop()
        state = self.principal.get_current_state()
        self.assertEqual(state, 'ST_IDLE')
        self.assertEqual(len(self.principal.dl_query), 1)
        #self.gaplic.mt_process()
        self.gaplic._loop()
        self.gaplic._loop()
        self.assertEqual(len(self.principal.dl_query), 0)



# -*- encoding: utf-8 -*-
"""
GObj :class:`GConnex`
=====================

GObj for manage socket connection handshake.

.. autoclass:: GConnex
    :members: start_up, get_next_dst

"""
from collections import deque

from ginsfsm.gobj import GObj
from ginsfsm.c_timer import GTimer
from ginsfsm.c_sock import GSock


def ac_drop(self, event):
    self.send_event(self._gsock, 'EV_DROP')


def ac_timeout_disconnected(self, event):
    if self._disabled:
        return
    if self.timeout_inactivity > 0:
        pass  # don' connect until arrives data to transmit
    else:
        self.connect()


def ac_disconnected(self, event):
    if not self._disabled:
        self.set_timeout(self.timeout_between_connections)
    if self.disconnected_event_name is not None:
        event.event_name = self.disconnected_event_name
        self.broadcast_event(event)


def ac_timeout_wait_connected(self, event):
    self.set_timeout(self.timeout_between_connections)


def ac_connected(self, event):
    self.clear_timeout()
    if self.timeout_inactivity > 0:
        self.set_timeout(self.timeout_inactivity)
    self.process_dl_tx_data()
    if self.connected_event_name is not None:
        event.event_name = self.connected_event_name
        self.broadcast_event(event)


def ac_rx_data(self, event):
    if self.timeout_inactivity > 0:
        self.set_timeout(self.timeout_inactivity)
    if self.rx_data_event_name is not None:
        event.event_name = self.rx_data_event_name
        self.broadcast_event(event)


def ac_timeout_data(self, event):
    self.send_event(self._gsock, 'EV_DROP')


def ac_tx_data(self, event):
    if self.timeout_inactivity > 0:
        self.set_timeout(self.timeout_inactivity)
    self.send_event(self._gsock, 'EV_SEND_DATA', data=event.kw['data'])


def ac_enqueue_tx_data(self, event):
    self._dl_tx_data.append(event)
    # try to connect, if this function called, is because we are disconnected.
    self.connect()


def ac_transmit_ready(self, event):
    if self.transmit_ready_event_name is not None:
        event.event_name = self.transmit_ready_event_name
        self.broadcast_event(event)

CONNEX_FSM = {
    'event_list': (
        'EV_DROP:top input:bottom output',
        'EV_SEND_DATA:top input:bottom output ST_CONNECTED',
        'EV_CONNECTED:bottom input:top output',
        'EV_DISCONNECTED:bottom input:top output',
        'EV_TIMEOUT:bottom input',
        'EV_RX_DATA:bottom input:top output',
        'EV_TRANSMIT_READY:bottom input:top output',
    ),
    'state_list': (
        'ST_DISCONNECTED',
        'ST_WAIT_CONNECTED',
        'ST_CONNECTED'
    ),
    'machine': {
        'ST_DISCONNECTED':
        (
            ('EV_SEND_DATA',      ac_enqueue_tx_data,        None),
            ('EV_TIMEOUT',        ac_timeout_disconnected,   None),
        ),
        'ST_WAIT_CONNECTED':
        (
            ('EV_SEND_DATA',      ac_enqueue_tx_data,        None),
            ('EV_CONNECTED',      ac_connected,              'ST_CONNECTED'),
            ('EV_DISCONNECTED',   ac_disconnected,           'ST_DISCONNECTED'),
            ('EV_TIMEOUT',        ac_timeout_wait_connected, 'ST_DISCONNECTED'),
        ),
        'ST_CONNECTED':
        (
            ('EV_SEND_DATA',      ac_tx_data,                None),
            ('EV_DROP',           ac_drop,                   None),
            ('EV_DISCONNECTED',   ac_disconnected,           'ST_DISCONNECTED'),
            ('EV_TIMEOUT',        ac_timeout_data,           None),
            ('EV_RX_DATA',        ac_rx_data,                None),
            ('EV_TRANSMIT_READY', ac_transmit_ready,         None),
        ),
    }
}

CONNEX_GCONFIG = {
    'subscriber': [None, None, 0, None,
        "subcriber of all output-events."
        "Default is ``None``, i.e., the parent"
        ],
    'disabled': [bool, False, 0, None, "Set True to disabled the connection"],
    'destinations': [list, [('', 0)], 0, None,
        "list of destination (host,port) tuples."],
    'timeout_waiting_connected': [int, 60, 0, None, ""],
    'timeout_between_connections': [int, 5, 0, None,
        "Idle timeout to wait between attempts of connection."],
    'timeout_inactivity': [int, -1, 0, None,
        "Inactivity timeout to close the connection."
        "Reconnect when new data arrived. With -1 never close."
        ],

    # If some name is None then parent don't want receive it.
    'connected_event_name': [str, 'EV_CONNECTED', 0, None,
        "Name of the *connected* event."
        " ``None`` if you want ignore the event"
        ],
    'disconnected_event_name': [str, 'EV_DISCONNECTED', 0, None,
        "Name of the *disconnected* event."
        " ``None`` if you want ignore the event"
        ],
    'transmit_ready_event_name': [str, 'EV_TRANSMIT_READY', 0, None,
        "Name of the *transmit_ready* event."
        " ``None`` if you want ignore the event"
        ],
    'rx_data_event_name': [str, 'EV_RX_DATA', 0, None,
        "Name of the *rx_data* event."
        " ``None`` if you want ignore the event"
        ],
}


class GConnex(GObj):
    """  GConnex gobj.
    Responsible for maintaining the client socket connected, or not.
    It can maintain the connection closed, until new data arrived.
    It can have several destinations to connect.

    .. ginsfsm::
       :fsm: CONNEX_FSM
       :gconfig: CONNEX_GCONFIG

    *Input-Events:*
        * :attr:`'EV_SEND_DATA'`: transmit ``event.data``.

          Mandatory attributes of the received :term:`event`:

          * ``data``: data to send.

    *Output-Events:*
        * :attr:`'EV_CONNECTED'`: socket connected.

          Attributes added to the sent :term:`event`:

            * ``peername``: remote address to which the socket is connected.
            * ``sockname``: the socket’s own address.

        * :attr:`'EV_DISCONNECTED'`: socket disconnected.
        * :attr:`'EV_TRANSMIT_READY'`: socket ready to transmit more data.
        * :attr:`'EV_RX_DATA'`: data received.
          Attributes added to the sent :term:`event`:

            * ``data``: Data received from remote address.

    """
    def __init__(self):
        self._dl_tx_data = deque()  # queue for tx data.
        self._timer = None
        self._disabled = False
        #TODO: give access to _gsock properties: rx/tx msgs, etc
        self._gsock = None
        self._idx_dst = 0
        # warning: prevent overwrite_parameters before attrs are created.
        GObj.__init__(self, CONNEX_FSM, CONNEX_GCONFIG)

    def start_up(self):
        """ Initialization zone.

        Subcribe all enabled :term:`output-event`'s to ``subscriber``
        with this sentence::

            self.subscribe_event(None, self.subscriber)
        """
        self.subscribe_event(None, self.subscriber)
        if self.name and len(self.name):
            prefix_name = self.name
        else:
            prefix_name = None

        self._gsock = self.create_gobj(
            prefix_name + '.gsock' if prefix_name else None,
            GSock,
            self,
            #trace_dump=self.trace_dump, TODO usa gconfig
        )
        self._gsock.get_next_dst = self.get_next_dst

        self._timer = self.create_gobj(
            prefix_name + '.timer' if prefix_name else None,
            GTimer,
            self,
            timeout_event_name='EV_TIMEOUT')
        if not self._disabled:
            self.set_timeout(2)  # self.timeout_between_connections

    @property
    def disabled(self):
        return self._disabled

    @disabled.setter
    def disabled(self, value):
        self._disabled = value
        if value:
            self.set_timeout(-1)
            self.send_event(self._gsock, 'EV_DROP')
        else:
            self.set_timeout(2)

    def set_timeout(self, seconds):
        if self._timer:  # protect from overwrite_parameters
            self.send_event(self._timer, 'EV_SET_TIMER', seconds=seconds)

    def clear_timeout(self):
        if self._timer:  # protect from overwrite_parameters
            self.send_event(self._timer, 'EV_SET_TIMER', seconds=-1)

    def connect(self):
        self.set_new_state('ST_WAIT_CONNECTED')
        self.set_timeout(self.timeout_waiting_connected)
        self.send_event(self._gsock, 'EV_CONNECT')

    def get_next_dst(self):
        """ Return the destination (host,port) tuple to connect from
        the ``destinations`` attribute.
        If there are multiple tuples in ``destinations`` attribute,
        try to connect to each tuple cyclically.
        Override :meth:`ginsfsm.c_sock.GSock.get_next_dst`.
        """
        host, port = self.destinations[self._idx_dst]
        self._idx_dst += 1
        if self._idx_dst >= len(self.destinations):
            self._idx_dst = 0
        return (host, port)

    def process_dl_tx_data(self):
        while True:
            try:
                event = self._dl_tx_data.popleft()
            except IndexError:
                break
            else:
                self.send_event(
                    self._gsock,
                    'EV_SEND_DATA',
                    data=event.kw['data'])



# -*- coding: utf-8 -*-
import logging
from collections import (
    Callable,
    deque,
    )


class DeferredInterrupt(Exception):
    def __init__(self, deferred_ref):
        Exception.__init__(self)
        self.deferred_ref = deferred_ref


class Deferred(object):
    def __init__(self, ref, func, *args, **kwargs):
        """Return a new Deferred instance
        :param ref: Find a callback function by his reference.
        :param func: Deferred callback function.
        :param args: args to callback function.
        :param kwargs: kwargs to callback function.
        """
        assert isinstance(func, Callable)
        self.ref = ref
        self.func = func
        self.args = args or []
        self.kwargs = kwargs or {}

    def __call__(self, **kwargs):
        """Processing the callback .
        """
        kw = self.kwargs.copy()
        kw.update(kwargs)
        return self.func(*self.args, **kw)


class DeferredList(object):
    """ List of deferred callbacks
        Add a callable (function or method).
    """
    def __init__(self):
        self.callbacks = deque()

    def add_callback(self, ref, func, *args, **kwargs):
        new_deferred = Deferred(ref, func, *args, **kwargs)
        self.callbacks.append(new_deferred)
        return new_deferred

    def __call__(self, ref, **kwargs):
        for deferred in self.callbacks:
            if deferred.ref == ref:
                return deferred(**kwargs)
        logging.error("ERROR __call__ deferred ref %r NOT FOUND" % deferred.ref)

    def delete(self, ref):
        for deferred in self.callbacks:
            if deferred.ref == ref:
                self.callbacks.remove(deferred)
                return
        logging.error("ERROR delete deferred ref %r NOT FOUND" % deferred.ref)



# -*- encoding: utf-8 -*-

import time
import traceback
import logging

from .parser import HTTPRequestParser
from .task import (
    ErrorTask,
    WSGITask,
    )
from .utilities import (
    InternalServerError,
    )

from ginsfsm.gobj import GObj
from ginsfsm.deferred import DeferredInterrupt


def ac_execute_requests(self, event):
    """Execute all pending requests """
    requests = event.requests
    gsock = self.gsock
    ext_event = None
    if hasattr(event, 'ext_event'):
        ext_event = event.ext_event
    while requests:
        request = requests[0]
        if request.error:
            task = ErrorTask(self.wsgi_server, gsock, request)
        else:
            # Añado esto en get_environment de task.py,despues de environ = {}
            # environ['gobj.gsock'] = gsock
            task = WSGITask(self.wsgi_server, gsock, request, ext_event)
        try:
            task.service()
        except DeferredInterrupt as e:

            def callback(request, ext_event=None):
                self.send_event(
                    self,
                    'EV_EXECUTE_REQUESTS',
                    requests=[request],
                    ext_event=ext_event,
                )
            self.gaplic.deferred_list.add_callback(
                e.deferred_ref,
                callback,
                request
            )
        except:
            logging.exception('Exception when serving %s' %
                                  task.request.path)
            if not task.wrote_header:
                if self.expose_tracebacks:
                    body = traceback.format_exc()
                else:
                    body = ('The server encountered an unexpected '
                            'internal server error')
                request = HTTPRequestParser(
                    url_scheme=self.wsgi_server.url_scheme)
                request.error = InternalServerError(body)
                task = ErrorTask(self.wsgi_server, gsock, request)
                task.service()  # must not fail
            else:
                task.close_on_finish = True
        # we cannot allow self.requests to drop to empty til
        # here; otherwise the mainloop gets confused
        if task.close_on_finish:
            gsock.close_when_flushed = True
            for request in requests:
                request._close()
            requests = []
        else:
            request = requests.pop(0)
            request._close()

    gsock.send_some()
    gsock.last_activity = time.time()


WSGIAPP_FSM = {
    'event_list': (
        'EV_EXECUTE_REQUESTS: top input',
        'EV_WSGIAPP_RESPONSE: top output',
    ),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_EXECUTE_REQUESTS', ac_execute_requests,    'ST_IDLE'),
        ),
    }
}

WSGIAPP_GCONFIG = {
    'wsgi_server': [None, None, 0, None, "wsgi server"],
    'gsock': [None, None, 0, None, "clisrv gsock"],
    'expose_tracebacks': [bool, False, 0, None,
        "expose tracebacks of uncaught exceptions"],
}


class WSGIApp(GObj):
    """  Execute WSGI apps.
    """

    def __init__(self):
        GObj.__init__(self, WSGIAPP_FSM, WSGIAPP_GCONFIG)

    def start_up(self):
        pass



import unittest
from ginsfsm.globals import set_global_app
from ginsfsm.gobj import (
    GObj,
    EventNotAcceptedError,
    EventError,
    StateError,
    DestinationError,
    GObjError,
    QueueError,
)

import logging
logging.basicConfig(level=logging.DEBUG)


########################################################
#       Machine samples to test filters
########################################################
#******************************
#       Transmitter
#******************************
def ac_pulse(self, event):
    self.pulses_emitted += 1
    self.broadcast_event('EV_WAVEOUT', origin=self.pulses_emitted)

FSM_TRANSMITTER = {
    'event_list': (
        'EV_PULSE',
        'EV_WAVEOUT:output',
    ),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_PULSE', ac_pulse, None),
        ),
    }
}


def event_filter(value_returned_by_action):
    if value_returned_by_action is True:
        return True
    return False


def app():
    return


def app_factory(global_config, **local_conf):
    return app

TRANSMITTER_GCONFIG = [{
    'param1': [int, 0, 0, None, ""],
    'param2': [bool, False, 0, None, ""],
    'param3': [str, 'app:xxx', 0, None, ""],
    'param4': [str, 'app:ginsfsm.tests.test_gobj2.app_factory', 0, None, ""],
}]

set_global_app('ginsfsm.tests.test_gobj2.app_factory', app)


class TransmitterGObj(GObj):
    def __init__(self):
        GObj.__init__(self, FSM_TRANSMITTER, TRANSMITTER_GCONFIG)

    def start_up(self):
        ''' Create gobj childs, initialize something...
        '''
        self.pulses_emitted = 0
        self.set_owned_event_filter(event_filter)


#******************************
#       Receptor
#******************************

def ac_wave(self, event):
    if event.origin == self.mymod:
        print("    MINE %s! pulse %s" % (self.name, event.origin))
        self.count += 1
        return True
    return False

RECEPTOR_FSM = {
    'event_list': ('EV_WAVEOUT',),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_WAVEOUT', ac_wave, None),
        ),
    }
}


RECEPTOR_GCONFIG = [{
    'transmitter': [None, None, 0, None, ""],
    'mymod': [int, 0, 0, None, ""],
}]


class ReceptorGObj(GObj):
    def __init__(self):
        GObj.__init__(self, RECEPTOR_FSM, RECEPTOR_GCONFIG)
        """ Attributes"""
        #self.transmitter = None
        #self.mymod = 0
        """ End Attributes"""
        self.count = 0

    def start_up(self):
        ''' Create gobj childs, initialize something...
        '''
        self.transmitter.subscribe_event('EV_WAVEOUT', self)


#******************************
#       Principal
#******************************

class PrincipalGObj(GObj):
    def __init__(self):
        GObj.__init__(self, {})
        self.start_up()

    def start_up(self):
        ''' Create gobj childs, initialize something...
        '''
        self.transmitter = self.create_gobj('Transmitter', TransmitterGObj, self)
        self.receptor_list = list(range(4))
        for idx in list(range(4)):
            self.receptor_list[idx] = self.create_gobj('Receptor%d' % (idx + 1),
                ReceptorGObj, self, transmitter=self.transmitter, mymod=idx + 1)

        settings = {
            'GObj.trace_mach': True,
            'GObj.logger': logging,
        }
        self.overwrite_parameters(-1, **settings)
        #for idx in range(4):
        #    self.send_event(self.transmitter, 'EV_PULSE')


########################################################
#       Tests
########################################################


class TestGObj2(unittest.TestCase):
    def setUp(self):
        self.principal = PrincipalGObj()

    def tearDown(self):
        self.principal.destroy_gobj(self.principal.transmitter)

    def test_owned_event_filter(self):
        self.principal.send_event(self.principal.transmitter, 'EV_PULSE')
        self.assertEqual(self.principal.receptor_list[0].count, 1)
        self.assertEqual(self.principal.receptor_list[1].count, 0)
        self.assertEqual(self.principal.receptor_list[2].count, 0)
        self.assertEqual(self.principal.receptor_list[3].count, 0)
        self.principal.send_event(self.principal.transmitter, 'EV_PULSE')
        self.assertEqual(self.principal.receptor_list[0].count, 1)
        self.assertEqual(self.principal.receptor_list[1].count, 1)
        self.assertEqual(self.principal.receptor_list[2].count, 0)
        self.assertEqual(self.principal.receptor_list[3].count, 0)
        self.principal.send_event(self.principal.transmitter, 'EV_PULSE')
        self.assertEqual(self.principal.receptor_list[0].count, 1)
        self.assertEqual(self.principal.receptor_list[1].count, 1)
        self.assertEqual(self.principal.receptor_list[2].count, 1)
        self.assertEqual(self.principal.receptor_list[3].count, 0)
        self.principal.send_event(self.principal.transmitter, 'EV_PULSE')
        self.assertEqual(self.principal.receptor_list[0].count, 1)
        self.assertEqual(self.principal.receptor_list[1].count, 1)
        self.assertEqual(self.principal.receptor_list[2].count, 1)
        self.assertEqual(self.principal.receptor_list[3].count, 1)

    def test_travesal_pyramid(self):
        self.assertRaises(KeyError, self.principal.__getitem__, None)
        self.assertRaises(KeyError, self.principal.__getitem__, 'XXX')
        gobj = self.principal.__getitem__('Transmitter')

    def test_overwrite_parameters(self):
        gobj = self.principal.__getitem__('Transmitter')
        settings = {
            'Transmitter.param1': 2,
            'TransmitterGObj.param2': True,
            'TransmitterGObj.param3': 'app:yyy',
        }
        gobj.overwrite_parameters(-1, **settings)
        self.assertTrue(gobj.read_parameter('param1') == 2)
        self.assertTrue(gobj.read_parameter('param2') == True)
        self.assertTrue(gobj.read_parameter('param3') == None)
        self.assertTrue(gobj.read_parameter('param4') == app)

        settings = {
            'Transmitter.param4': 'xxx',
        }
        gobj.overwrite_parameters(-1, **settings)
        self.assertTrue(gobj.read_parameter('param4') == app)



import unittest
from ginsfsm.gmsg import GMsg


class TestGMsg(unittest.TestCase):
    def setUp(self):
        self.gmsg = GMsg(3, 5)

    def test_getdata_half(self):
        self.gmsg.putdata("123ab")
        data = self.gmsg.getdata(3)
        self.assertEqual(data, b"123")
        data = self.gmsg.getdata(3)
        self.assertEqual(data, b"ab")
        data = self.gmsg.getdata(3)
        self.assertEqual(data, None)

        data = self.gmsg.reset_rd()
        data = self.gmsg.getdata(3)
        self.assertEqual(data, b"123")

    def test_getdata_full(self):
        self.gmsg.putdata("1234567890abcde")
        data = self.gmsg.getdata(3)
        self.assertEqual(data, b"123")
        data = self.gmsg.getdata(3)
        self.assertEqual(data, b"45")
        data = self.gmsg.getdata(3)
        self.assertEqual(data, None)

    def test_getchar(self):
        self.gmsg.putdata("123ab")
        for char in "123ab":
            self.assertEqual(self.gmsg.getchar(), ord(char))
        self.assertEqual(self.gmsg.getchar(), None)

    def test_check_char(self):
        self.gmsg.putdata("123ab")
        for char in "123ab":
            self.assertEqual(self.gmsg.check_char(), ord('1'))

    def test_cur_seg_rd(self):
        self.gmsg.putdata("123ab")
        data = self.gmsg.cur_seg_rd()
        self.assertEqual(data, bytearray(b"123"))

    def test_insert_data_full(self):
        self.gmsg.putdata("123ab")
        ret = self.gmsg.insert_data("x")
        self.assertFalse(ret)

    def test_insert_data_half(self):
        self.gmsg.putdata("12")
        ret = self.gmsg.insert_data("x")
        self.assertTrue(ret)

        data = self.gmsg.cur_seg_rd()
        self.assertEqual(data, bytearray(b"x12"))
        self.assertEqual(data, b"x12")

    def test_mark_and_overwrite(self):
        self.gmsg.putdata("12")
        self.gmsg.mark_pos_write()
        self.gmsg.putdata("3abx")
        marked_bytes_len = self.gmsg.marked_bytes()
        self.assertEqual(marked_bytes_len, 3)

        self.gmsg.overwrite_data("qwerty")
        data = self.gmsg.getdata(20)
        self.assertEqual(data, b"12qwe")
        marked_count = self.gmsg.marked_bytes()
        self.assertEqual(marked_count, 0)

    def test_bytesleft(self):
        self.gmsg.putdata("12")
        count = self.gmsg.bytesleft()
        self.assertEqual(count, 2)
        self.gmsg.putdata("abcd")
        count = self.gmsg.bytesleft()
        self.assertEqual(count, 5)
        self.gmsg.getdata(2)
        count = self.gmsg.bytesleft()
        self.assertEqual(count, 3)
        self.gmsg.getdata(12)
        count = self.gmsg.bytesleft()
        self.assertEqual(count, 0)

    def test_totalbytes(self):
        self.gmsg.putdata("12")
        count = self.gmsg.totalbytes()
        self.assertEqual(count, 2)
        self.gmsg.putdata("abcd")
        count = self.gmsg.bytesleft()
        self.assertEqual(count, 5)



import unittest

from ginsfsm.gconfig import (
    GConfig,
    GConfigTemplateError,
    GConfigValidateError,
    )


class TestGConfig(unittest.TestCase):
    def setUp(self):
        self.gconfig = GConfig({})

    def test_set_bad_argument(self):
        self.assertRaises(GConfigTemplateError, GConfig, '')
        self.assertRaises(GConfigTemplateError, GConfig, {0: []})
        self.assertRaises(GConfigTemplateError, GConfig, {'': []})
        self.assertRaises(GConfigTemplateError, GConfig, {'x': []})
        self.assertRaises(GConfigTemplateError, GConfig, {'x': 0})
        self.assertRaises(GConfigTemplateError, GConfig, {'x': [0]})
        self.assertRaises(GConfigTemplateError, GConfig,
                          {'x': [0, 0, 0, 0, '']})
        self.assertRaises(GConfigTemplateError, GConfig,
                          {'x': [GConfig, 0, 0, None, '']})

        def p():
            pass
        self.assertRaises(GConfigTemplateError, GConfig, {'x': [0, 0, 0, p, 0]})

        conf = GConfig({'x': [int, 0, 0, None, '']})
        self.assertTrue(isinstance(conf, GConfig))
        self.assertTrue(hasattr(conf, 'x'))
        value = conf.read_parameter('x')
        self.assertEqual(value, 0)

        conf = GConfig({'y': [int, -2, 0, None, '']})
        self.assertTrue(isinstance(conf, GConfig))
        self.assertTrue(hasattr(conf, 'y'))
        value = conf.read_parameter('y')
        self.assertEqual(value, -2)

        conf = GConfig({
            'bt': [bool, "True", 0, None, ''],
            'bf': [bool, "False", 0, None, ''],
            'bn': [bool, None, 0, None, ''],
            'x': [int, 100, 0, None, ''],
            'y': [int, -2, 0, None, ''],
            's': [str, "kk", 0, None, ''],
            'None': [str, None, 0, None, ''],
            }
        )
        self.assertTrue(isinstance(conf, GConfig))
        value = conf.read_parameter('x')
        self.assertEqual(value, 100)
        value = conf.read_parameter('y')
        self.assertEqual(value, -2)
        value = conf.read_parameter('s')
        self.assertEqual(value, 'kk')
        value = conf.read_parameter('bt')
        self.assertEqual(value, True)
        value = conf.read_parameter('bf')
        self.assertEqual(value, False)
        value = conf.read_parameter('bn')
        self.assertEqual(value, False)
        value = conf.read_parameter('None')
        self.assertEqual(value, None)

        conf.write_parameters(bt=False)
        value = conf.read_parameter('bt')
        self.assertEqual(value, False)

        conf.reset_parameter('bt')
        value = conf.read_parameter('bt')
        self.assertEqual(value, True)

        conf = GConfig({
            'lst': [list, ['', 0], 0, None, ''],
            'tup': [tuple, ('', 0), 0, None, ''],
            }
        )
        self.assertTrue(isinstance(conf, GConfig))
        value = conf.read_parameter('lst')
        self.assertEqual(value, ['', 0])
        value = conf.read_parameter('tup')
        self.assertEqual(value, ('', 0))
        conf.write_parameters(xx=False)

        definition = [tuple, ('', 0), 0, None, '']
        conf = GConfig({
            'tup': definition,
            }
        )
        conf.write_parameters(tup=(0, 0))
        definition[0] = GConfig
        conf.write_parameters(tup=(0, 0))

        conf = GConfig({
            'lst': [list, ['', 0], 0, None, ''],
            'tup': [tuple, ('', 0), 0, None, ''],
            }
        )
        conf.read_parameters()

        conf = GConfig({
            'x': [None, GConfig, 0, None, ''],
            }
        )
        value = conf.read_parameter('x')
        self.assertTrue(value is GConfig)

        def validate(value):
            if value < -5:
                raise Exception
            if value < 0:
                return False
            return True

        self.assertRaises(GConfigValidateError, GConfig,
            {'y': [int, -10, 0, validate, '']})
        value = conf.read_parameter('y')
        self.assertTrue(value is None)

        conf = GConfig({'y': [int, 10, 0, validate, '']})
        value = conf.read_parameter('y')
        self.assertTrue(value is 10)
        conf.write_parameters(y=20)
        value = conf.read_parameter('y')
        self.assertTrue(value is 20)
        conf.reset_all_parameters()
        value = conf.read_parameter('y')
        self.assertTrue(value is 10)
        value = conf.read_parameter('yyy')
        self.assertTrue(value is None)



import unittest

from ginsfsm.gaplic import GAplic
from ginsfsm.gobj import GObj

from ginsfsm.c_timer import GTimer

import logging
logging.basicConfig(level=logging.DEBUG)


def ac_timeout(self, event):
    self.timeout_done = True


FSM = {
    'event_list': ('EV_TIMEOUT',),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_TIMEOUT',      ac_timeout,     None),
        ),
    }
}


class GPrincipal(GObj):
    """  Principal gobj.
    """
    def __init__(self):
        GObj.__init__(self, FSM)
        self.timer = None
        self.timeout_done = False

    def start_up(self):
        """ Create child and start the timer.
        """
        self.timer = self.create_gobj(
            'timer',
            GTimer,
            self,
            timeout_event_name='EV_TIMEOUT')

        settings = {
            'GObj.trace_mach': True,
            'GObj.logger': logging,
        }
        self.overwrite_parameters(-1, **settings)

    def set_timeout(self, seconds):
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=seconds)

    def clear_timeout(self):
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=-1)


class TestGTimer(unittest.TestCase):
    def setUp(self):
        self.ga = GAplic(name='Test')
        self.principal = self.ga.create_gobj('principal', GPrincipal, None)

    def test_send_event(self):
        self.principal.set_timeout(2)

        while self.ga._loop():
            pass
        self.assertTrue(self.principal.timeout_done is True)


class MyGAplic(GAplic):
    dones = [0] * 3

    def mt_subprocess(self):
        i = 0
        self.principal = self.search_unique_gobj('principal')
        for done in self.dones:
            if done:
                i += 1
                continue

            if i == 0:
                self.principal.set_timeout(2)
                self.dones[i] = True

            elif i == 1:
                if self.principal.timeout_done is True:
                    self.dones[i] = True

            elif i == 2:
                exit()

            i += 1

if __name__ == "__main__":
    ga = MyGAplic('TestGAplic')
    ga.create_gobj('principal', GPrincipal, None)
    try:
        ga.mt_process()
    except KeyboardInterrupt:
        print('Program stopped')



# -*- encoding: utf-8 -*-

import os
import sys

from setuptools import setup, find_packages

if sys.version_info[:2] < (2, 6):
    raise RuntimeError('Requires Python 2.6 or better')

here = os.path.abspath(os.path.dirname(__file__))
try:
    README = open(os.path.join(here, 'README.rst')).read()
    CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()
except IOError:
    README = CHANGES = ''

from ginsfsm import __version__

setup(name='ginsfsm',
    version=__version__,
    description='GinsFSM, a library to develop systems based in '
                'finite-state machines',
    long_description=README + "\n\n",  # + CHANGES,
    classifiers=[
        "Intended Audience :: Developers",
        "Development Status :: 4 - Beta",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Communications",
        "Topic :: System :: Networking",
        "License :: OSI Approved :: MIT License",
    ],

    keywords='framework communication finite state machine fsm '
             'socket poll epoll kqueue select server client workflow',
    author='ginsmar',
    author_email='ginsmar at artgins.com',
    url='http://ginsfsm.org',
    license='MIT License',
    packages=find_packages(exclude=[]),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'PasteDeploy >= 1.5.0',  # py3 compat
    ],
    extras_require=dict(
        rst=[
            'docutils',
            'svgwrite',
            'pillow',
        ],
    ),
    tests_require=[],
    test_suite="ginsfsm.tests",
    entry_points="""
        [ginsfsm.scaffold]
        simple_gobj=ginsfsm.scaffolds:SimpleGObjTemplate

        [console_scripts]
        gcreate = ginsfsm.scripts.gcreate:main
        gserve = ginsfsm.scripts.gserve:main
    """,
    )
